/** ============================================================================
 *  @file   _sync_usr.c
 *
 *  @path   $(DSPLINK)/gpp/src/api/PrOS/
 *
 *  @desc   Implementation of sub-component SYNC for user-side.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

/*  ----------------------------------- OS Specific Headers         */
#include <itron.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>

/*  ----------------------------------- OSAL Headers                */
#include <sync.h>
#include <mem_os.h>

/*  ----------------------------------- User API                    */
#include <_idm_usr.h>
#include <_sync_usr.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_API_SYNC

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#define SET_FAILURE_REASON  \
            TRC_3PRINT (TRC_LEVEL7, \
                        "\nFailure: Status:[0x%x] File:[0x%x] Line:[%d]\n", \
                        status, FID_C_API_SYNC, __LINE__)

/** ============================================================================
 *  @const  MAX_SYNC_CS
 *
 *  @desc   Maximum number of CS objects that can be created.
 *  ============================================================================
 */
#define MAX_SYNC_CS               512u


/** ============================================================================
 *  @name   SYNC_USR_CsObject_tag
 *
 *  @desc   This object is used by various CS API's.
 *          The same object defined by the OSAL is used here.
 *  ============================================================================
 */
typedef SyncCsObject SYNC_USR_CsObject_tag ;


/** ============================================================================
 *  @name   SYNC_USR_State
 *
 *  @desc   State object for the SYNC_USR component
 *
 *  @field  isInitialized
 *              Indicates whether the SYNC_USR component is initialized.
 *  @field  csObjs
 *              Array of pointers to SYNC_USR_CsObject objects.
 *  ============================================================================
 */
typedef struct SYNC_USR_State_tag {
    Bool                isInitialized ;
    SYNC_USR_CsObject * csObjs [MAX_SYNC_CS] ;
} SYNC_USR_State ;


/** ============================================================================
 *  @name   SYNC_USR_stateObj
 *
 *  @desc   State object containing all global information for the SYNC_USR
 *          component.
 *  ============================================================================
 */
STATIC SYNC_USR_State SYNC_USR_stateObj ;


/** ============================================================================
 *  @func   _SYNC_USR_init
 *
 *  @desc   Initialize SYNC USR sub-component by allocating all resources.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_SYNC_USR_init (Void)
{
    DSP_STATUS  status = DSP_SOK ;
    IDM_Attrs   idmAttrs ;
    Uint32      i ;

    TRC_0ENTER ("_SYNC_USR_init") ;

    /* OSAL_Initialize () is done by PROC_setup (). */

    /* Use the component ID as the unique key for the IDM component. */
    idmAttrs.baseId = 0 ;
    idmAttrs.maxIds = MAX_SYNC_CS ;
    status = _IDM_USR_create (COMPONENT_ID, &idmAttrs) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    for (i = 0 ; i < MAX_SYNC_CS ; i++) {
        SYNC_USR_stateObj.csObjs [i] = NULL ;
    }

    SYNC_USR_stateObj.isInitialized = TRUE ;

    TRC_1LEAVE ("_SYNC_USR_init", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_exit
 *
 *  @desc   Releases resources used by the sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_SYNC_USR_exit ()
{
    DSP_STATUS  status = DSP_SOK ;
    Uint32      i ;

    TRC_0ENTER ("_SYNC_USR_exit") ;

    for (i = 0 ; i < MAX_SYNC_CS ; i++) {
        SYNC_USR_stateObj.csObjs [i] = NULL ;
    }

    /* Delete the IDM object created for the SYNC_USR component. */
    status = _IDM_USR_delete (COMPONENT_ID) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    SYNC_USR_stateObj.isInitialized = FALSE ;

    /* OSAL_Finalize () is done by PROC_destroy (). */

    TRC_1LEAVE ("_SYNC_USR_exit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_createCS
 *
 *  @desc   Creates the Critical section structure.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_SYNC_USR_createCS (IN Pstr idKey, IN OUT SYNC_USR_CsObject ** csObj)
{
    DSP_STATUS          status = DSP_SOK ;
    Uint32              id ;


    TRC_2ENTER ("_SYNC_USR_createCS", idKey, csObj) ;

    DBC_Require (idKey != NULL) ;
    DBC_Require (csObj != NULL) ;

    if ((idKey == NULL) || (csObj == NULL)) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        /* Check if a new object is being created, or a handle to existing
         * object is to be returned.
         */
        status = _IDM_USR_acquireId (COMPONENT_ID, idKey, &id) ;
        if (status == DSP_SEXISTS) {
            /* Return the handle to existing CS object at specified ID. */
            DBC_Assert (SYNC_USR_stateObj.csObjs [id] != NULL) ;
            *csObj = SYNC_USR_stateObj.csObjs [id] ;
            status = DSP_SEXISTS ;
        }
        else if (DSP_SUCCEEDED (status)) {
            status = SYNC_CreateCS ((SyncCsObject **) csObj) ;
            if (DSP_SUCCEEDED (status)) {
                /* Store the handle to the CS object for the specified ID. */
                SYNC_USR_stateObj.csObjs [id] = *csObj ;
            }
            else {
                SET_FAILURE_REASON ;
            }
        }
        else {
            SET_FAILURE_REASON ;
        }
    }

    DBC_Ensure (    ((csObj  == NULL) && DSP_FAILED (status))
                 || ((*csObj == NULL) && DSP_FAILED (status))
                 || ((*csObj != NULL) && DSP_SUCCEEDED (status))) ;

    TRC_1LEAVE ("_SYNC_USR_createCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_deleteCS
 *
 *  @desc   Deletes the critical section object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_SYNC_USR_deleteCS (IN OUT SYNC_USR_CsObject ** csObj)
{
    DSP_STATUS status = DSP_SOK ;
    Uint32     id ;

    TRC_1ENTER ("_SYNC_USR_deleteCS", csObj) ;

    DBC_Require (csObj != NULL) ;

    if (csObj == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else if (*csObj == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        SYNC_ProtectionStart () ;

        /* Search for the specified csObj in array of existing CS objects to
         * get its ID.
         */
        for (id = 0 ; id < MAX_SYNC_CS ; id++) {
            if (SYNC_USR_stateObj.csObjs [id] == *csObj) {
                /* Found the CS object. */
                status = _IDM_USR_releaseId (COMPONENT_ID, id) ;
                if (status == DSP_SFREE) {
                    /* If the last handle to the CS object is being deleted,
                     * delete the SYNC CS object.
                     */
                    status = SYNC_DeleteCS ((SyncCsObject *) *csObj) ;
                    *csObj = NULL ;
                    SYNC_USR_stateObj.csObjs [id] = NULL ;
                    if (DSP_SUCCEEDED (status)) {
                        status = DSP_SFREE ;
                    }
                    else {
                        SET_FAILURE_REASON ;
                    }
                }
                break ;
            }
        }

        SYNC_ProtectionEnd () ;
    }

    TRC_1LEAVE ("_SYNC_USR_deleteCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_enterCS
 *
 *  @desc   This function enters the critical section that is passed as
 *          argument to it. After successful return of this function no other
 *          process can enter until this process exits the CS by calling
 *          _SYNC_USR_leaveCS.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_SYNC_USR_enterCS (IN SYNC_USR_CsObject * csObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("_SYNC_USR_enterCS", csObj) ;

    DBC_Require (csObj != NULL) ;

    if (csObj == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status = SYNC_EnterCS ((SyncCsObject *) csObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("_SYNC_USR_enterCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_leaveCS
 *
 *  @desc   This function makes critical section available for other processes
 *          to enter.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_SYNC_USR_leaveCS (IN SYNC_USR_CsObject * csObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("_SYNC_USR_leaveCS", csObj) ;

    DBC_Require (csObj != NULL) ;

    if (csObj == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status = SYNC_LeaveCS ((SyncCsObject *) csObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("_SYNC_USR_leaveCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_sleep
 *
 *  @desc   This function yields control to other tasks for the defined time
 *          interval. Delay in milliseconds is provided.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
_SYNC_USR_sleep (IN Uint32 delay)
{

    if (delay != 0) {
        dly_tsk ((TMO)delay) ;
    }

    return DSP_SOK ;
}


/** ============================================================================
 *  @func   _SYNC_USR_stateObjInit
 *
 *  @desc   Initialize SYNC USR sub-component by allocating all resources.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
_SYNC_USR_stateObjInit (Void)
{
    DSP_STATUS  status = DSP_SOK ;
    Uint32      i ;

    TRC_0ENTER ("_SYNC_USR_init") ;

    for (i = 0 ; i < MAX_SYNC_CS ; i++) {
        SYNC_USR_stateObj.csObjs [i] = NULL ;
    }
    SYNC_USR_stateObj.isInitialized = TRUE ;

    TRC_1LEAVE ("_SYNC_USR_stateObjInit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   _SYNC_USR_setPriority
 *
 *  @desc   Change the priority of the current  task.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS _SYNC_USR_setPriority (IN Uint32 priority)
{
    DSP_STATUS   status   = DSP_SOK;

    TRC_1ENTER ("_SYNC_USR_setPriority", priority) ;

    status = SYNC_SetPriority (priority);

    TRC_1LEAVE ("_SYNC_USR_setPriority", status) ;

    return status;
}


/** ============================================================================
 *  @func   _SYNC_USR_getPriority
 *
 *  @desc   Get the priority of the current  task.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS _SYNC_USR_getPriority (OUT Uint32 * curPri)
{
    DSP_STATUS   status   = DSP_SOK;
 
	TRC_1ENTER ("_SYNC_USR_getPriority", curPri) ;

	status = SYNC_GetPriority (curPri);

	TRC_1LEAVE ("_SYNC_USR_getPriority", status) ;
 
	return status;	
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
