/** ============================================================================
 *  @file   drv_api.c
 *
 *  @path   $(DSPLINK)/gpp/src/api/PrOS/
 *
 *  @desc   Implemetation of wrapper ioctl function.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#if defined (CHNL_COMPONENT)
#include <chnldefs.h>
#endif /* if defined (CHNL_COMPONENT) */
#if defined (POOL_COMPONENT)
#include <pooldefs.h>
#endif /* if defined (POOL_COMPONENT) */
#if defined (MPLIST_COMPONENT)
#include <mplistdefs.h>
#endif  /* if defined (MPLIST_COMPONENT) */
#if defined (RINGIO_COMPONENT)
#include <ringiodefs.h>
#endif  /* if defined (RINGIO_COMPONENT) */
#if defined (MPCS_COMPONENT)
#include <mpcsdefs.h>
#endif  /* if defined (MPCS_COMPONENT) */

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>

/*  ----------------------------------- OSAL Headers                */
#include <mem_os.h>
#include <mem.h>
#include <drvdefs.h>
#include <drv_api.h>

/*  ----------------------------------- API                         */
#if defined (POOL_COMPONENT)
#include <_pool.h>
#endif /* if defined (POOL_COMPONENT) */
#include <_proc.h>
#include <_sync_usr.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_OSAL_DRV

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  \
            TRC_3PRINT (TRC_LEVEL7, \
                        "\nFailure: Status:[0x%x] File:[0x%x] Line:[%d]\n", \
                        status, FID_C_OSAL_DRV_API, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @name   DRV_Object_tag
 *
 *  @desc   OS specific definition of the driver object.
 *
 *  @field  signature
 *              Signature of the object.
 *  @field  refCount
 *              Reference count for the driver object.
 *  ============================================================================
 */
struct DRV_Object_tag {
    Uint32   signature    ;
    Uint32   refCount     ;
} ;

/*  ----------------------------------------------------------------------------
 *  @name   PROC_stateObj
 *
 *  @desc   State object for the PROC user-level component containing internal
 *          state information required by it.
 *  ----------------------------------------------------------------------------
 */
extern PROC_Object PROC_stateObj ;


/** ============================================================================
 *  @name   DRV_Initialize
 *
 *  @desc   Initialization function for the driver.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DRV_Initialize (OUT DRV_Object ** drvObj,     IN OUT Pvoid arg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_2ENTER ("DRV_Initialize", drvObj, arg) ;

    DBC_Require (drvObj != NULL) ;

    if (drvObj == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (*drvObj != NULL) {
        status = _SYNC_USR_enterCS (PROC_stateObj.syncCsObj) ;
        if (DSP_SUCCEEDED (status)) {
            (*drvObj)->refCount++ ;
            status = _SYNC_USR_leaveCS (PROC_stateObj.syncCsObj) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
        else {
            SET_FAILURE_REASON ;
        }
    }
    else {
        status = DRV_PMGR_Initialize () ;
        if (DSP_SUCCEEDED (status)) {
            status = MEM_Calloc ((Void **)drvObj,
                                 sizeof (DRV_Object),
                                 MEM_DEFAULT) ;
            if (DSP_SUCCEEDED (status)) {
                (*drvObj)->signature = SIGN_DRV ;
                (*drvObj)->refCount  = 1 ;
            }
            else {
                status = DSP_EMEMORY ;
                SET_FAILURE_REASON ;
            }
        }
        else {
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("DRV_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @name   DRV_Finalize
 *
 *  @desc   Finalize function for the driver.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DRV_Finalize (IN DRV_Object * drvObj,     IN OUT Pvoid arg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_2ENTER ("DRV_Finalize", drvObj, arg) ;

    DBC_Require (drvObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (drvObj, SIGN_DRV) == TRUE) ;

    if (IS_OBJECT_VALID (drvObj, SIGN_DRV)) {
        /* This is called in protection from proc_detach, and for proc_destroy
         * It is the last entity calling DRV_Finalize.
         */
        if( drvObj->refCount > 0 ) {
            drvObj->refCount-- ;
        }

        if (drvObj->refCount == 0) {

            status = FREE_PTR (drvObj) ;
            drvObj = NULL ;

            if (DSP_FAILED (status)) {
                status = DSP_EMEMORY ;
                SET_FAILURE_REASON ;
            }
            else {
                status = DRV_PMGR_Finalize () ;
                if (DSP_FAILED (status)) {
                    SET_FAILURE_REASON ;
                }
                else {
                    status = DSP_SFINALIZED ;
                }
            }
        }
    }
    else {
         status = DSP_EPOINTER ;
         SET_FAILURE_REASON ;
    }
    TRC_1LEAVE ("DRV_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @name   DRV_Invoke
 *
 *  @desc   Invokes the lower layer function, which is OS dependent.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DRV_Invoke (IN         DRV_Object * drvObj,
            IN         Uint32       cmdId,
                IN OUT Pvoid        arg1,
                IN OUT Pvoid        arg2)
{
    DSP_STATUS  status   = DSP_SOK ;
    CMD_Args *  args     = arg1    ;

    TRC_4ENTER ("DRV_Invoke", drvObj, cmdId, arg1, arg2) ;

    switch (cmdId) {
    case CMD_PROC_SETUP:
        {
            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
#if defined (POOL_COMPONENT)
            else {
                _POOL_init () ;
            }
#endif /* if defined (POOL_COMPONENT) */
        }
        break ;

    case CMD_PROC_DESTROY:
        {
#if defined (POOL_COMPONENT)
            _POOL_exit () ;
#endif /* if defined (POOL_COMPONENT) */

            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
        break ;

    case CMD_PROC_ATTACH:
        {
            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }

            /* This is the case in which thread of an already attached process
             * tries to attach.
             */
            if (   (DSP_SUCCEEDED (status))
                && (args->apiStatus == DSP_SALREADYATTACHED)) {
                args->apiStatus = DSP_EALREADYCONNECTED ;
            }
        }
        break ;

#if defined (POOL_COMPONENT)
    case CMD_POOL_OPEN:
        {
            POOL_OpenParams * retParams ;
            Uint8            poolNo ;
            ProcessorId      procId ;

            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_SUCCEEDED (status)) {
                /* Get the proc Number for the processor */
                procId = POOL_getProcId (args->apiArgs.poolOpenArgs.poolId) ;
                /* Get the pool Number on the processor */
                poolNo = POOL_getPoolNo (args->apiArgs.poolOpenArgs.poolId) ;
                retParams = args->apiArgs.poolOpenArgs.params ;

                POOL_addrConfig [procId][poolNo].addr [AddrType_Usr] =
                                                    retParams->physAddr ;
                POOL_addrConfig [procId][poolNo].addr [AddrType_Phy] =
                                                    retParams->physAddr ;
                POOL_addrConfig [procId][poolNo].addr [AddrType_Knl] =
                                                    retParams->physAddr ;
                POOL_addrConfig [procId][poolNo].addr [AddrType_Dsp] =
                                                    retParams->dspAddr ;
                POOL_addrConfig [procId][poolNo].size     = retParams->size ;
                POOL_addrConfig [procId][poolNo].isInit   = TRUE ;
            }
            else {
                SET_FAILURE_REASON ;
            }
        }
        break ;

    case CMD_POOL_CLOSE:
        {
            Uint8            poolNo ;
            ProcessorId      procId ;

            /* Get the proc Number for the processor */
            procId = POOL_getProcId (args->apiArgs.poolOpenArgs.poolId) ;
            /* Get the pool Number on the processor */
            poolNo = POOL_getPoolNo (args->apiArgs.poolOpenArgs.poolId) ;
                /* Check if pool was opened */
            if (    POOL_addrConfig [procId][poolNo].addr [AddrType_Usr]
                    ==  (Uint32) NULL) {
                status = DSP_EINVALIDARG ;
            }
            else {
                status = DRV_CallAPI (cmdId, args) ;
                if (DSP_SUCCEEDED (status)) {
                    POOL_addrConfig [procId][poolNo].addr [AddrType_Usr] =
                        (Uint32) NULL ;
                    POOL_addrConfig [procId][poolNo].addr [AddrType_Phy] =
                        (Uint32) NULL ;
                    POOL_addrConfig [procId][poolNo].addr [AddrType_Knl] =
                        (Uint32) NULL ;
                    POOL_addrConfig [procId][poolNo].addr [AddrType_Dsp] =
                        (Uint32) NULL ;
                    POOL_addrConfig [procId][poolNo].size = 0xFFFFFFFFu ;
                }
                else {
                    SET_FAILURE_REASON ;
                }
            }
        }
        break ;

    case CMD_POOL_TRANSLATEADDR:
        {
            POOL_AddrXltFlag xltFlag ;

            args->apiStatus = DSP_SOK ;

            xltFlag = (POOL_AddrXltFlag)
                    (  args->apiArgs.poolTranslateAddrArgs.srcAddrType
                     | (args->apiArgs.poolTranslateAddrArgs.dstAddrType << 8)) ;
            *(args->apiArgs.poolTranslateAddrArgs.dstAddr) =
                               args->apiArgs.poolTranslateAddrArgs.srcAddr ;
            _POOL_xltBuf (args->apiArgs.poolTranslateAddrArgs.poolId,
                          args->apiArgs.poolTranslateAddrArgs.dstAddr,
                          xltFlag) ;

            if ((*(args->apiArgs.poolTranslateAddrArgs.dstAddr)) == NULL) {
                status = DSP_ENOTFOUND ;
                SET_FAILURE_REASON ;
            }
        }
        break ;
#endif /* if defined (POOL_COMPONENT) */

#if defined (RINGIO_COMPONENT)
    case CMD_RINGIO_MAPREGION:
        {
            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_SUCCEEDED (status)) {
                args->apiArgs.ringIoArgs.ringioRegionArgs.userAddr =
                            args->apiArgs.ringIoArgs.ringioRegionArgs.physAddr ;
            }
            else {
                SET_FAILURE_REASON ;
            }
        }
        break ;

        case CMD_RINGIO_UNMAPREGION:
            {
                /* Nothing to be done for unmapping the RingIO region */
                args->apiStatus = DSP_SOK ;
            }
            break ;
#endif /* if defined (RINGIO_COMPONENT) */

#if defined (MPLIST_COMPONENT)
    case CMD_MPLIST_MAPREGION:
        {
            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_SUCCEEDED (status)) {
                args->apiArgs.mplistArgs.mplistRegionArgs.userAddr =
                            args->apiArgs.mplistArgs.mplistRegionArgs.physAddr ;
            }
            else {
                SET_FAILURE_REASON ;
            }
        }
        break ;

        case CMD_MPLIST_UNMAPREGION:
            {
                /* Nothing to be done for unmapping the MPLIST region */
                args->apiStatus = DSP_SOK ;
            }
            break ;
#endif /* if defined (MPLIST_COMPONENT) */

#if defined (MPCS_COMPONENT)
    case CMD_MPCS_MAPREGION:
        {
            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_SUCCEEDED (status)) {
                args->apiArgs.mpcsMapArgs.mpcsRegionArgs.userAddr =
                            args->apiArgs.mpcsMapArgs.mpcsRegionArgs.physAddr ;
            }
            else {
                SET_FAILURE_REASON ;
            }
        }
        break ;

        case CMD_MPCS_UNMAPREGION:
            {
                /* Nothing to be done for unmapping the MPCS region */
                args->apiStatus = DSP_SOK ;
            }
            break ;
#endif /* if defined (MPCS_COMPONENT) */

    default:
        {
            status = DRV_CallAPI (cmdId, args) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
        break ;
    }

    if (DSP_SUCCEEDED (status)) {
        status = args->apiStatus ;
    }

    TRC_1LEAVE ("DRV_Invoke", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
