/** ============================================================================
 *  @file   drv_api.h
 *
 *  @path   $(DSPLINK)/gpp/src/api/PrOS/
 *
 *  @desc   User side driver wrapper interface.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


#if !defined (DRV_API_H)
#define DRV_API_H


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @name   DRV_Object_tag
 *
 *  @desc   Forward declaration for the OS specific driver object type.
 *  ============================================================================
 */
typedef struct DRV_Object_tag DRV_Object ;


/** ============================================================================
 *  @name   DRV_handle
 *
 *  @desc   Handle to global driver object.
 *  ============================================================================
 */
extern DRV_Object * DRV_handle ;

/** ============================================================================
 *  @macro  DRV_PROTECT_INIT
 *
 *  @desc   OS dependent definition of protection function.
 *          This function initailizes driver protection mechanism between
 *          multiple processes.
 *  ============================================================================
 */
#define DRV_PROTECT_INIT(handle)          (DSP_SOK)

/** ============================================================================
 *  @macro  DRV_PROTECT_EXIT
 *
 *  @desc   OS dependent definition of protection function.
 *          This function finalizes driver protection mechanism between
 *          multiple processes.
 *  ============================================================================
 */
#define DRV_PROTECT_EXIT(handle)          (DSP_SOK)

/** ============================================================================
 *  @macro  DRV_PROTECT_ENTER
 *
 *  @desc   OS dependent definition of protection function.
 *          This function enters protection between multiple processes.
 *  ============================================================================
 */
#define DRV_PROTECT_ENTER(handle)         (DSP_SOK)

/** ============================================================================
 *  @macro  DRV_PROTECT_LEAVE
 *
 *  @desc   OS dependent definition of protection function.
 *          This function leaves protection between multiple processes.
 *  ============================================================================
 */
#define DRV_PROTECT_LEAVE(handle)         (DSP_SOK)

/** ============================================================================
 *  @macro  DRV_SET_CURSTATUS
 *
 *  @desc   OS dependent definition of protection function.
 *          This function sets the status of DSPLink components.
 *  ============================================================================
 */
#define DRV_SET_CURSTATUS(curStatus)      (curStatus = TRUE)

/** ============================================================================
 *  @macro  DRV_RESET_CURSTATUS
 *
 *  @desc   OS dependent definition of protection function.
 *          This function resets the status of DSPLink components.
 *  ============================================================================
 */
#define DRV_RESET_CURSTATUS(curStatus)    (curStatus = FALSE)

/** ============================================================================
 *  @macro  DRV_CHECK_CURSTATUS
 *
 *  @desc   OS dependent definition of protection function.
 *          This function returns information about whether the specified
 *          curstatus value is set.
 *  ============================================================================
 */
#define DRV_CHECK_CURSTATUS(curStatus)    (curStatus == TRUE)

/** ============================================================================
 *  @macro  DRV_STARTUP_INIT
 *
 *  @desc   OS dependent definition of startup initialization functionality.
 *          This function performs all initialization required for startup.
 *  ============================================================================
 */
#define DRV_STARTUP_INIT(handle)          (DSP_SOK)

/** ============================================================================
 *  @name   DRV_Initialize
 *
 *  @desc   Initialization function for the driver.
 *
 *  @arg    drvObj
 *              pointer to driver object to be initialized.
 *  @arg    arg
 *              optional argument to be used for initialization.
 *
 *  @ret    DSP_SOK
 *              Operation successfully completed.
 *          DSP_EPOINTER
 *              Invalid pointer.
 *          DSP_EMEMORY
 *              Out of Memory error.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  drvObj must be a valid pointer.
 *
 *  @leave  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DRV_Initialize (IN OUT DRV_Object ** drvObj,     IN OUT Pvoid arg) ;


/** ============================================================================
 *  @name   DRV_Finalize
 *
 *  @desc   Finalization function for the driver.
 *
 *  @arg    drvObj
 *              Pointer to driver object to be finalized.
 *  @arg    arg
 *              Optional argument to be used for finalization.
 *
 *  @ret    DSP_SOK
 *              Operation successfully completed.
 *          DSP_EMEMORY
 *              Out of Memory error.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  drvObj must be a valid pointer.
 *
 *  @leave  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DRV_Finalize (IN DRV_Object * drvObj,     IN OUT Pvoid arg) ;


/** ============================================================================
 *  @name   DRV_Invoke
 *
 *  @desc   Trap to invoke API of PMGR layer.
 *
 *  @arg    drvObj
 *              Pointer to driver.
 *  @arg    cmdId
 *              Command identifier.
 *  @arg    arg1
 *              Optional argument 1.
 *  @arg    arg2
 *              Optional argument 2.
 *
 *  @ret    DSP_SOK
 *              Operation successfully completed.
 *          DSP_ENOTFOUND
 *               Registered event not found.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  drvObj must be a valid pointer.
 *
 *  @leave  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DRV_Invoke (IN         DRV_Object * drvObj,
            IN         Uint32       cmdId,
                IN OUT Pvoid        arg1,
                IN OUT Pvoid        arg2) ;


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */


#endif /* !define (DRV_API_H) */
