/** ============================================================================
 *  @file   dm6437_phy_vlynq.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/DM6437/vlynq/PrOS/
 *
 *  @desc   Linux vlynq driver interface file.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <dm6437_hal.h>
#include <dm6437_hal_vlynq_boot.h>
#include <dm6437_hal_vlynq_intgen.h>
#include <dm6437_hal_vlynq_pwr.h>
#include <dm6437_hal_vlynq_map.h>
#include <dm6437_hal_vlynq_dma.h>
#include <dm6437_hal_vlynq_rdwr.h>
#include <dm6437_phy_vlynq.h>


#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_VLYNQ_PHY

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON      TRC_SetReason (status, FID_C_ARCH_PHY_VLYNQ, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @const  VLYNQ_CONTROL_BASE
 *
 *  @desc   VLYNQ Control Base Address.
 *  ============================================================================
 */
#define VLYNQ_CONTROL_BASE      0x01C9D000u

/** ============================================================================
 *  @const  VLYNQ_LOACL_CTRL
 *
 *  @desc   VLYNQ Local Control Address.
 *  ============================================================================
 */
#define VLYNQ_LOACL_CTRL        (VLYNQ_CONTROL_BASE + 0x04u)

/** ============================================================================
 *  @const  VLYNQ_LOACL_CTRL/VLYNQ_LOCAL_INTPTR/VLYNQ_LOCAL_STAT
 *
 *  @desc   VLYNQ Local Control Address/Local Interrupt Pointer/Local status reg
 *  ============================================================================
 */
#define VLYNQ_LOCAL_CTRL        (VLYNQ_CONTROL_BASE + 0x04u)
#define VLYNQ_LOCAL_INTPTR      (VLYNQ_CONTROL_BASE + 0x18u)
#define VLYNQ_LOCAL_STAT        (VLYNQ_CONTROL_BASE + 0x08u)

/** ============================================================================
 *  @const  VLYNQ_LOCAL_XAM / VLYNQ_LOCAL_RAMS1/ VLYNQ_LOCALE_RAMO1
 *
 *  @desc   Mapping registers.
 *  ============================================================================
 */
#define VLYNQ_LOCAL_XAM         (VLYNQ_CONTROL_BASE + 0x1Cu)
#define VLYNQ_LOCAL_RAMS1       (VLYNQ_CONTROL_BASE + 0x20u)
#define VLYNQ_LOCAL_RAMO1       (VLYNQ_CONTROL_BASE + 0x24u)

/** ============================================================================
 *  @const  VLYNQ_PEER_CTRL/VLYNQ_PEER_INTPTR
 *
 *  @desc   VLYNQ Peer Control Address/Peer Interrupt Pointer.
 *  ============================================================================
 */
#define VLYNQ_PEER_CTRL         (VLYNQ_CONTROL_BASE + 0x84u)
#define VLYNQ_PEER_INTPTR       (VLYNQ_CONTROL_BASE + 0x98u)

/** ============================================================================
 *  @const  VLYNQ_LOCAL_XAM / VLYNQ_LOCAL_RAMS1/ VLYNQ_LOCALE_RAMO1
 *
 *  @desc   Mapping registers.
 *  ============================================================================
 */
#define VLYNQ_PEER_XAM          (VLYNQ_CONTROL_BASE + 0x9Cu)
#define VLYNQ_PEER_RAMS1        (VLYNQ_CONTROL_BASE + 0xA0u)
#define VLYNQ_PEER_RAMO1        (VLYNQ_CONTROL_BASE + 0xA4u)
#define VLYNQ_PEER_RAMS2        (VLYNQ_CONTROL_BASE + 0xA8u)
#define VLYNQ_PEER_RAMO2        (VLYNQ_CONTROL_BASE + 0xACu)
#define VLYNQ_PEER_RAMS3        (VLYNQ_CONTROL_BASE + 0xB0u)
#define VLYNQ_PEER_RAMO3        (VLYNQ_CONTROL_BASE + 0xB4u)
#define VLYNQ_PEER_RAMS4        (VLYNQ_CONTROL_BASE + 0xB8u)
#define VLYNQ_PEER_RAMO4        (VLYNQ_CONTROL_BASE + 0xBCu)


/** ============================================================================
 *  @const  LOCAL_VLYNQ_TXADR_MAP / LOCAL_VLYNQ_REMOTE_ADDR..
 *
 *  @desc   Local Vlynq Addresss mappings, Here we create 1 entries as follows.
 *          SInce we do not expect DSP to write into GPP memory.
 *  ============================================================================
 */
/* Local VLYNQ write address space, i.e. we write into this region to write to
 * peer.
 */
#define LOCAL_VLYNQ_TXADR_MAP   (0x0C000000u)
/* Address which is visiable to peer, default points to DSPLINK SHMEM */
#define LOCAL_VLYNQ_REMOTE_ADDR (0x67E00000u)
/* Local Region 1 */
#define LOCAL_REGION_1_ADDR     (LOCAL_VLYNQ_REMOTE_ADDR)
#define LOCAL_REGION_1_SIZE     (0x004000000u) /* 64MB */


/** ============================================================================
 *  @const  PEER_VLYNQ_TXADR_MAP / PEER_VLYNQ_REMOTE_ADDR..
 *
 *  @desc   Peer Vlynq Addresss mappings, Here we create 4 entries as follows.
 *          1. Entry -> DSPLINK Shared memory region.
 *          2. Entry -> Code region & Used as scratch.
 *          3. Entry -> Used for DMA.
 *          4. Entry -> User defined region.
 *  ============================================================================
 */
/* Peer VLYNQ write address space, i.e. peer write into this region to write to
 * local.
 */
#define PEER_VLYNQ_TXADR_MAP   (0x4C000000u)
/* Address which is visiable to local, default points to DSPLINK SHMEM */
#define PEER_VLYNQ_REMOTE_ADDR (0x80100000u)
/* Peer Region 1 */
#define PEER_REGION_1_ADDR     (PEER_VLYNQ_REMOTE_ADDR)
#define PEER_REGION_1_SIZE     (0x00100000u)
/* Peer Region 2 */
#define PEER_REGION_2_ADDR     (0x80000000u)
#define PEER_REGION_2_SIZE     (0x00100000u)
/* Peer Region 3 CFG registers */
#define PEER_REGION_3_ADDR     (0x001800000u)
#define PEER_REGION_3_SIZE     (0x000700000u)
/* Peer Region 4 */
#define PEER_REGION_4_ADDR     0x80200000/*(PEER_REGION_2_ADDR + PEER_REGION_2_SIZE)*/
#define PEER_REGION_4_SIZE     (0x0033FF000u) /* Remaining */


/** ============================================================================
 *  @name   DM6437HAL_vlynqInterface
 *
 *  @desc   Interface functions exported by the VLYNQ subcomponent.
 *  ============================================================================
 */
HAL_Interface DM6437_vlynqInterface = {
                                       &DM6437_phyVlynqInit,
                                       &DM6437_phyVlynqExit,
                                       &DM6437_halVlynqBootCtrl,
                                       &DM6437_halVlynqIntCtrl,
                                      NULL,
                                       &DM6437_halVlynqMapCtrl,
                                       &DM6437_halVlynqPwrCtrl,
                                       &DM6437_halVlynqRead,
                                       &DM6437_halVlynqWrite,
                                       &DM6437_halVlynqReadDMA,
                                       &DM6437_halVlynqWriteDMA,
                                      } ;


/* ============================================================================
 *  @func   DM6437_phyVlynqInit
 *
 *  @desc   Initializes VLYNQ device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DM6437_phyVlynqInit (IN Pvoid halObj)
{
    DSP_STATUS      status    = DSP_SOK ;
    DM6437_HalObj * halObject = (DM6437_HalObj *) halObj ;

    TRC_1ENTER ("DM6437_phyVlynqInit", halObj) ;

    DBC_Require (halObj != NULL) ;

    if (halObj != NULL) {
        /* Initialize the VLYNQ */
        REG(VLYNQ_LOCAL_CTRL)   = 0x4727df84u ;
        REG(VLYNQ_LOCAL_INTPTR) = 0x01C9D014u ;
        REG(VLYNQ_PEER_CTRL)    = 0x47070004u ;
        REG(VLYNQ_PEER_INTPTR)  = 0x01E01014u ;

        /* Configure the default MAP settings for local */
        REG(VLYNQ_LOCAL_RAMO1) = LOCAL_REGION_1_ADDR ;
        REG(VLYNQ_LOCAL_RAMS1) = LOCAL_REGION_1_SIZE ;
        /* Local Transmitter address */
        REG(VLYNQ_LOCAL_XAM)   = LOCAL_VLYNQ_TXADR_MAP ;

        /* Configure the default MAP settings for peer */
        REG(VLYNQ_PEER_RAMO1)  = PEER_REGION_1_ADDR ;
        REG(VLYNQ_PEER_RAMS1)  = PEER_REGION_1_SIZE ;
        REG(VLYNQ_PEER_RAMO2)  = PEER_REGION_2_ADDR ;
        REG(VLYNQ_PEER_RAMS2)  = PEER_REGION_2_SIZE ;
        REG(VLYNQ_PEER_RAMO3)  = PEER_REGION_3_ADDR ;
        REG(VLYNQ_PEER_RAMS3)  = PEER_REGION_3_SIZE ;
        REG(VLYNQ_PEER_RAMO4)  = PEER_REGION_4_ADDR ;
        REG(VLYNQ_PEER_RAMS4)  = PEER_REGION_4_SIZE ;
        /* Peer Transmitter address */
        REG(VLYNQ_PEER_XAM)    = 0x0u ;

        /* Wait for VLYNQ to be up */
        while (!(REG(VLYNQ_LOCAL_STAT) & 0x1u)) {
        }

        /* VLYNQ is up here and usable */
        halObject->vlynq.region1Addr = LOCAL_VLYNQ_TXADR_MAP ;
        halObject->vlynq.region1Size = PEER_REGION_1_SIZE   ;
        halObject->vlynq.region2Addr = (  LOCAL_VLYNQ_TXADR_MAP
                                        + PEER_REGION_1_SIZE) ;
        halObject->vlynq.region2Size = PEER_REGION_2_SIZE   ;
        halObject->vlynq.region3Addr = (  LOCAL_VLYNQ_TXADR_MAP
                                        + PEER_REGION_1_SIZE
                                        + PEER_REGION_2_SIZE) ;
        halObject->vlynq.region3Size = PEER_REGION_3_SIZE   ;
        halObject->vlynq.region4Addr = (  LOCAL_VLYNQ_TXADR_MAP
                                        + PEER_REGION_1_SIZE
                                        + PEER_REGION_2_SIZE
                                        + PEER_REGION_3_SIZE) ;
        halObject->vlynq.region4Size  = PEER_REGION_4_SIZE   ;
        halObject->vlynq.intVector    = 31u   ;
        halObject->vlynq.ctrlBaseAddr = VLYNQ_CONTROL_BASE   ;

         /* Create the lock for DMA resource */
        halObject->lock = SYNC_SpinLockCreate () ;
    }
    else {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("DM6437_phyVlynqInit", status) ;

    return status ;
}


/* ============================================================================
 *  @func   DM6437_phyVlynqExit
 *
 *  @desc   Finalizes VLYNQ device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DM6437_phyVlynqExit (IN Pvoid halObj)
{
    DSP_STATUS          status    = DSP_SOK ;
    DM6437_HalObj *     halObject = (DM6437_HalObj *) halObj ;

    TRC_1ENTER ("DM6437_phyVlynqInit", halObj) ;

    DBC_Require (halObject != NULL) ;

    SYNC_SpinLockDelete (halObject->lock) ;

    /* Nothing to done here */

    TRC_1LEAVE ("DM6437_phyVlynqInit", status) ;

    return status ;
}


#if defined (__cplusplus)

}
#endif
