/** ============================================================================
 *  @file   ldrv_mpcs_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/ldrv/PrOS/
 *
 *  @desc   Implementation of OS-specific functionality for LDRV.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <print.h>
#include <mem.h>
#include <sync.h>

/*  ----------------------------------- Link Driver                 */
#include <ldrv_mpcs_os.h>


#if defined (__cplusplus)
extern "C" {
#endif

/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_LDRV_MPCS_OS

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_LDRV_MPCS_OS, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


#if defined (GPP_TSK_MODE)
/** ============================================================================
 *  @name   MPCS_SEMCREATED_OPEN
 *
 *  @desc   Value to indicate that the SEM object created in _MPCS_open.
 *  ============================================================================
 */
#define MPCS_SEMCREATED_OPEN            0xABABABAB

/** ============================================================================
 *  @name   MPCS_SEMCREATED_CREATE
 *
 *  @desc   Value to indicate that the SEM object created in MPCS_create.
 *  ============================================================================
 */
#define MPCS_SEMCREATED_CREATE          0xCDCDCDCD

/** ============================================================================
 *  @name   MPCS_GlobalLock
 *
 *  @desc   Global Lock for MPCS critical section protection
 *  ============================================================================
 */
Uint32 MPCS_GlobalLock = 0     ;
#endif



/** ============================================================================
 *  @func   LDRV_MPCS_OS_moduleInit
 *
 *  @desc   This function initializes the LDRV_MPCS module.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
LDRV_MPCS_OS_moduleInit (Void)
{
    DSP_STATUS         status = DSP_SOK ;
#if defined (GPP_TSK_MODE)
    SyncCsObject * csObj      = NULL ;

#endif

    TRC_0ENTER ("LDRV_MPCS_OS_moduleInit") ;

#if defined (GPP_TSK_MODE)
    if (MPCS_GlobalLock == 0) {
        status = SYNC_CreateCS (&(csObj)) ;
        MPCS_GlobalLock = (Uint32)(csObj);
    }
#endif

    TRC_1LEAVE ("LDRV_MPCS_OS_moduleInit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_MPCS_OS_moduleExit
 *
 *  @desc   This function finalizes the LDRV_MPCS module.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
LDRV_MPCS_OS_moduleExit (Void)
{
    DSP_STATUS status    = DSP_SOK ;
#if defined (GPP_TSK_MODE)
    SyncCsObject * csObj   = NULL ;

#endif

    TRC_0ENTER ("LDRV_MPCS_OS_moduleExit") ;

#if defined (GPP_TSK_MODE)
    csObj  = (SyncCsObject *)MPCS_GlobalLock ;
    status = SYNC_DeleteCS ((SyncCsObject *)csObj) ;
    MPCS_GlobalLock = 0 ;
#endif

    TRC_1LEAVE ("LDRV_MPCS_OS_moduleExit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_MPCS_OS_create
 *
 *  @desc   This function creates and initializes an instance of the MPCS
 *          object.
 *          The memory for the object may or may not be provided by the user.
 *          If provided by the user, the memory for the object must be shared
 *          across the processors using the MPCS. It must also already be mapped
 *          into user space for OSes supporting user/kernel separation.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
LDRV_MPCS_OS_create (IN     ProcessorId    dspId,
                     IN     MPCS_ShObj *   mpcsObj)
{
    DSP_STATUS status = DSP_SOK ;


    TRC_2ENTER ("LDRV_MPCS_OS_create", dspId, mpcsObj) ;

    DBC_Require (IS_VALID_PROCID (dspId)) ;
    DBC_Require (mpcsObj != NULL) ;

#if defined (GPP_TSK_MODE)
        if (MPCS_GlobalLock != 0) {
            mpcsObj->gppMpcsObj.localLock = MPCS_GlobalLock ;
            mpcsObj->gppMpcsObj.localFlag = MPCS_SEMCREATED_CREATE ;
        }
        else {
            status = DSP_EHANDLE ;
            SET_FAILURE_REASON ;
        }
#else
    mpcsObj->gppMpcsObj.priority      = 0 ;
    mpcsObj->dspMpcsObj.priority      = 0 ;
#endif /* if defined(GPP_TSK_MODE) */


    TRC_1LEAVE ("LDRV_MPCS_OS_create", status) ;

    return status ;
}



/** ============================================================================
 *  @func   LDRV_MPCS_OS_delete
 *
 *  @desc   This function deletes the instance of the MPCS object.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
LDRV_MPCS_OS_delete (IN     ProcessorId  dspId,
                     IN     MPCS_ShObj * mpcsObj)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_2ENTER ("LDRV_MPCS_OS_delete", dspId, mpcsObj) ;

    DBC_Require (IS_VALID_PROCID (dspId)) ;
    DBC_Require (mpcsObj != NULL) ;

#if defined (GPP_TSK_MODE)
    mpcsObj->gppMpcsObj.localLock = 0 ;
#else
    mpcsObj->gppMpcsObj.priority      = 0 ;
    mpcsObj->dspMpcsObj.priority      = 0 ;
#endif

    TRC_1LEAVE ("LDRV_MPCS_OS_delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_MPCS_OS_enter
 *
 *  @desc   This function enters the critical section specified by the MPCS
 *          object.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
LDRV_MPCS_OS_enter (IN     MPCS_ShObj * mpcsObj)
{
    DSP_STATUS status = DSP_SOK ;
#if defined(GPP_SWI_MODE)
    Uint32        curTskPriority     = 0 ;
    Uint32        userSpecPriority   = 0 ;
#endif

    TRC_1ENTER ("LDRV_MPCS_OS_enter", mpcsObj) ;

    DBC_Require (mpcsObj != NULL) ;

#if defined(GPP_TSK_MODE)
        status = SYNC_EnterCS ((SyncCsObject *)mpcsObj->gppMpcsObj.localLock) ;
#else
       if (GPP_SWI_MODE_PRI == 0xffff) {
            userSpecPriority = 1 ;
        }
        else {
            userSpecPriority = GPP_SWI_MODE_PRI ;
        }

        status = SYNC_GetPriority (&curTskPriority) ;

        if (DSP_SUCCEEDED (status)) {
            mpcsObj->gppMpcsObj.priority = curTskPriority ;
            status = SYNC_SetPriority (userSpecPriority);
        }
        else {
            SET_FAILURE_REASON ;
        }
#endif /* if defined (GPP_TSK_MODE) */

    TRC_1LEAVE ("LDRV_MPCS_OS_enter", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_MPCS_OS_leave
 *
 *  @desc   This function leaves the critical section specified by the MPCS
 *          object.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
LDRV_MPCS_OS_leave (IN     MPCS_ShObj * mpcsObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("LDRV_MPCS_OS_leave", mpcsObj) ;

    DBC_Require (mpcsObj != NULL) ;

#if defined(GPP_TSK_MODE)
    status = SYNC_LeaveCS ((SyncCsObject *)mpcsObj->gppMpcsObj.localLock) ;
#else
    status = SYNC_SetPriority (mpcsObj->gppMpcsObj.priority);
#endif /* if defined (GPP_TSK_MODE) */

    TRC_1LEAVE ("LDRV_MPCS_OS_leave", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif
