/** ============================================================================
 *  @file   dpc.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implementation of sub-component DPC.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <itron.h>
#include <kernel.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and sub-component Identifier.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_DPC

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_DPC, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif

/** ============================================================================
 *  @const  MAX_DPC
 *
 *  @desc   Maximum number of DPCs supported in DSP/BIOS Link port to PrOS.
 *  ============================================================================
 */
#define MAX_DPC             16u

/** ============================================================================
 *  @const  INVALID_INDEX
 *
 *  @desc   Invalid index of a DPC object.
 *  ============================================================================
 */
#define INVALID_INDEX       0xFFFFFFFF

/** ============================================================================
 *  @const  MAX_DPC_ACTCOUNT
 *
 *  @desc   Maximum activation request for a DPC.
 *  ============================================================================
 */
#define MAX_DPC_ACTCOUNT    0xFFFFFFFF

/** ============================================================================
 *  @const  DPC_PRIORITY
 *
 *  @desc   Priority value of tasks used for implementing DPC.
 *  ============================================================================
 */
#define DPC_PRIORITY 4u

/** ============================================================================
 *  @const  DPC_STACK_SIZE
 *
 *  @desc   Stack size of DPC calls.
 *  ============================================================================
 */
#define DPC_STACK_SIZE  1024u


/** ============================================================================
 *  @name   DpcObject_tag
 *
 *  @desc   PrOS specific definition of DpcObject.
 *          The type for this structure is defined in dpc.h as a forward
 *          declaration.
 *
 *  @field  signature
 *              Signature of the DPC object.
 *  @field  index
 *              Index of the DPC object.
 *  @field  actCount
 *              Activation count reference.
 *  @field  paramData
 *              Parameter data to be passed to the user defined deferred
 *              function call.
 *  @field  userDPCFn
 *              user defined function to be called as a deferred function call.
 *  @field  tskId
 *              ID number of task.
 *
 *  @see    DpcTasksMap
 *  ============================================================================
 */
struct DpcObject_tag {
    Uint32    signature    ;
    Uint32    index        ;
    volatile  Uint32    actCount     ;
    Pvoid     paramData    ;
    FnDpcProc userDPCFn    ;
    ID        tskId        ;
} ;


/** ============================================================================
 *  @name   DpcTasksMap
 *
 *  @desc   This structure defines the association between DpcObjects and their
 *          corresponding HISRs. It also contains a bitmap for tracking used
 *          Dpc objects.
 *
 *  @field  usedDPCs
 *              Bitmap for tracking used DPCs.
 *  @field  dpcs
 *              Global store for DPCs.
 *  @field  cTasks
 *              Array of PrOS task creation informations.
 *
 *  @see    DpcObject_tag
 *  ============================================================================
 */
typedef struct DpcTasksMap_tag {
    Uint32     usedDPCs         ;
    DpcObject  dpcs  [MAX_DPC]  ;
    T_CTSK     cTasks [MAX_DPC]  ;
} DpcTasksMap ;


/** ============================================================================
 *  @name   DPC_DpcTasksTable
 *
 *  @desc   Subcomponent variable to keep track of all the DPCs and associated
 *          hisrs created for 'Link.
 *  ============================================================================
 */
STATIC DpcTasksMap   DPC_DpcTasksTable ;


/** ============================================================================
 *  @name   DPC_IsInitialized
 *
 *  @desc   Flag to keep track of initialization of this subcomponent.
 *  ============================================================================
 */
STATIC Bool DPC_IsInitialized = FALSE ;

/** ============================================================================
 *  @name   DPC_IsEnabled
 *
 *  @desc   Flag to keep track whether execution of DPC's is enabled or disabled
 *  ============================================================================
 */
STATIC Bool DPC_IsEnabled = TRUE ;


/** ----------------------------------------------------------------------------
 *  @func   DPC_Callback
 *
 *  @desc   This function gets invoked by PrOS when it schedules an
 *          ISR to execute. It gets the currently running ISR, locates
 *          the associated DPC and invokes the user specified function.
 *
 *  @arg    None
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    DPC_Schedule
 *  ----------------------------------------------------------------------------
 */
void
DPC_Callback (VP_INT exinf) ;


/** ============================================================================
 *  @func   DPC_Initialize
 *
 *  @desc   This function initializes the DPC module. It initializes the
 *          global area (DPC_DpcTasksTable structure) for holding all the
 *          DPC objects and marks the UsedDPCs bitmap to indicate that no
 *          DPCs are currently in use.
 *
 *  @modif  DPC_DpcTasksTable
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Initialize ()
{
    DSP_STATUS  status   = DSP_SOK ;
    T_CTSK  *   tmpTask  = NULL    ;
    Uint16      index              ;

    TRC_0ENTER ("DPC_Initialize") ;

    if (DPC_IsInitialized == FALSE) {
        DPC_DpcTasksTable.usedDPCs = 0 ;

        for (index = 0 ; index < MAX_DPC ; index++) {
            tmpTask = &DPC_DpcTasksTable.cTasks [index] ;
            tmpTask->exinf   = (VP_INT) NULL  ;
            tmpTask->tskatr  = 0     ;
            tmpTask->task    = NULL  ;
            /* Lowest priority when initializing. This will be overriden at the
             * time of create of DPCs.
             */
            tmpTask->itskpri = 100   ;
            tmpTask->stksz   = 512   ;
            tmpTask->stk     = NULL  ;
        }

        DPC_IsInitialized = TRUE ;
    }

    TRC_1LEAVE ("DPC_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Finalize
 *
 *  @desc   This function provides an interface to exit from the DPC module.
 *          It frees up all the used DPCs and releases all the resources used by
 *          this module.
 *
 *  @modif  DPC_DpcTasksTable
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Finalize ()
{
    DSP_STATUS  status   = DSP_SOK ;
    ER          erStatus = E_OK    ;
    Uint32      i        = 0       ;
    DpcObject * dpcObj   = NULL    ;
    T_CTSK *    tmpTask  = NULL    ;

    TRC_0ENTER ("DPC_Finalize") ;

    DBC_Require (DPC_IsInitialized == TRUE) ;

    if (DPC_IsInitialized == TRUE) {
        /*
         * Get all the used DPCs objects from the usedDPC bitmask
         * and release them.
         */
        for (i = 0 ; i < MAX_DPC ; i++) {
            if (TEST_BIT (DPC_DpcTasksTable.usedDPCs, i) == 1u) {
                /*
                 *  Kill the associated task before resetting the DPC object.
                 */
                dpcObj = &DPC_DpcTasksTable.dpcs [i] ;
                erStatus = ter_tsk (dpcObj->tskId) ;

               /* Reset the task object. */
                if (erStatus == E_OK) {
                    tmpTask = &DPC_DpcTasksTable.cTasks [i] ;
                    tmpTask->exinf   = (VP_INT) NULL  ;
                    tmpTask->tskatr  = 0     ;
                    tmpTask->task    = NULL  ;
                    tmpTask->itskpri = 100   ;
                    tmpTask->stksz   = 512   ;
                    tmpTask->stk     = NULL  ;
                }
                else {
                    status = DSP_EFAIL ;
                    SET_FAILURE_REASON ;
                }
            }
        }
    }

    DPC_IsInitialized = FALSE ;
    DPC_DpcTasksTable.usedDPCs = 0 ;

    DBC_Ensure (DPC_IsInitialized == FALSE) ;
    DBC_Ensure (DPC_DpcTasksTable.usedDPCs == 0) ;

    TRC_1LEAVE ("DPC_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Create
 *
 *  @desc   Creates a DPC object and returns it after populating
 *          relevant fields. It sets a bit in UsedDPCs bitmask.
 *          It also creates a T_CTSK at corresponding index in
 *          DpcTasksMap array.
 *
 *  @modif  DPC_DpcTasksTable
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Create (IN  FnDpcProc    userDPCFn,
            IN  Pvoid        dpcArgs,
            OUT DpcObject ** dpcObj)
{
    DSP_STATUS  status   = DSP_SOK ;
    ER_ID       erId     = E_OK    ;
    Uint32      index    = 0       ;
    Bool        found    = FALSE   ;
    DpcObject * dpcObject = NULL   ;
    T_CTSK *    ctsk      = NULL   ;

    TRC_3ENTER ("DPC_Create", userDPCFn, dpcArgs, dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (userDPCFn != NULL) ;
    DBC_Require (dpcObj != NULL) ;

    if ((userDPCFn == NULL) || (dpcObj == NULL)) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        while ((found == FALSE) && (index < MAX_DPC)) {
            if (TEST_BIT (DPC_DpcTasksTable.usedDPCs, index) == 0u) {
                found = TRUE ;
            }
            else {
                index++ ;
            }
        }

        if (found == FALSE) {
            status = DSP_ERESOURCE ;
            SET_FAILURE_REASON ;
        }
        else {
            SET_BIT (DPC_DpcTasksTable.usedDPCs, index) ;
            dpcObject = &DPC_DpcTasksTable.dpcs   [index] ;
            ctsk      = &DPC_DpcTasksTable.cTasks [index] ;

            dpcObject->signature   = SIGN_DPC  ;
            dpcObject->index       = index     ;
            dpcObject->paramData   = dpcArgs   ;
            dpcObject->userDPCFn   = userDPCFn ;
            dpcObject->actCount    = 0         ;

            /*
             *  Initialize the T_CTSK structure.
             */
            ctsk->exinf   = (VP_INT) index    ;
            ctsk->tskatr  = TA_HLNG           ;
            ctsk->task    = (FP) DPC_Callback ;
            ctsk->itskpri = DPC_PRIORITY      ;
            ctsk->stksz   = DPC_STACK_SIZE  ;

            /* Allocate memory for stack */
            status = MEM_Alloc (
                              (Pvoid *) &(DPC_DpcTasksTable.cTasks [index].stk),
                              DPC_DpcTasksTable.cTasks [index].stksz,
                              MEM_DEFAULT);
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
    }

    if (DSP_SUCCEEDED (status)) {
        erId = acre_tsk(&DPC_DpcTasksTable.cTasks [index]);
        if (erId > 0) {
            DPC_DpcTasksTable.dpcs [index].tskId = erId ;
            *dpcObj = &DPC_DpcTasksTable.dpcs [index] ;
        }
        else {
            FREE_PTR(DPC_DpcTasksTable.cTasks [index].stk)         ;
            DPC_DpcTasksTable.dpcs [index].paramData   = NULL      ;
            DPC_DpcTasksTable.dpcs [index].userDPCFn   = NULL      ;
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }

    if ((DSP_FAILED (status)) && (dpcObj != NULL)) {
        *dpcObj = NULL ;
    }

    TRC_1LEAVE ("DPC_Create", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Delete
 *
 *  @desc   Destroys the DPC object by resetting the UsedDPCs bitmask.
 *
 *  @modif  DPC_DpcTasksTable
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Delete (IN DpcObject * dpcObj)
{
    DSP_STATUS status   = DSP_SOK ;
    ER         erStatus = E_OK    ;
    Uint32     index    = 0       ;
    T_RTSK     pk_rtsk            ;

    TRC_1ENTER ("DPC_Delete", dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (dpcObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (dpcObj, SIGN_DPC)) ;

    if (IS_OBJECT_VALID (dpcObj, SIGN_DPC) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        index = dpcObj->index ;
        if (index != INVALID_INDEX) {
            DBC_Assert (DPC_DpcTasksTable.dpcs [index].actCount == 0) ;

            CLEAR_BIT (DPC_DpcTasksTable.usedDPCs, index) ;

            /* Get the process info */
            erStatus = ref_tsk (DPC_DpcTasksTable.dpcs [index].tskId, &pk_rtsk);
            if (erStatus == E_OK) {
                if (pk_rtsk.tskstat != TTS_DMT) {
                    if (pk_rtsk.actcnt > 0) {
                        erStatus = can_act (
                                         DPC_DpcTasksTable.dpcs [index].tskId) ;
                        if (erStatus == E_NOEXS || erStatus == E_ID) {
                            status = DSP_EFAIL ;
                            SET_FAILURE_REASON ;
                        }
                    }

                    if (DSP_SUCCEEDED (status)) {
                        erStatus = ter_tsk (
                                         DPC_DpcTasksTable.dpcs [index].tskId) ;
                        if (erStatus != E_OK) {
                            status = DSP_EFAIL ;
                            SET_FAILURE_REASON ;
                        }
                    }
                }
            }
            else {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }

            erStatus = del_tsk (DPC_DpcTasksTable.dpcs [index].tskId) ;
            if (erStatus != E_OK) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            /* Reset the cTasks informations. */
            DPC_DpcTasksTable.dpcs [index].signature    = SIGN_NULL         ;
            DPC_DpcTasksTable.dpcs [index].index        = INVALID_INDEX     ;
            DPC_DpcTasksTable.dpcs [index].userDPCFn    = NULL      ;
            DPC_DpcTasksTable.dpcs [index].actCount     = 0 ;

            /* Deallocate memory block used for stack */
            FREE_PTR(DPC_DpcTasksTable.cTasks [index].stk);
        }
        else {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("DPC_Delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Cancel
 *
 *  @desc   Sets the Cancelled flag in DpcObj to true to indicate that the DPC
 *          has been cancelled.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Cancel (IN DpcObject * dpcObj)
{
    DSP_STATUS status   = DSP_SOK ;
    ER         erStatus = E_OK    ;
    ER_ID      erId     = E_OK    ;
    Uint32     index    = 0       ;
    T_RTSK     pk_rtsk            ;
    Uint32     irqFlags           ;

    TRC_1ENTER ("DPC_Cancel", dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (dpcObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (dpcObj, SIGN_DPC)) ;

    if (IS_OBJECT_VALID (dpcObj, SIGN_DPC) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        index  = dpcObj->index ;

        if (index != INVALID_INDEX) {
            irqFlags = SYNC_SpinLockStart () ;
            /* Get the process info */
            erStatus = ref_tsk (DPC_DpcTasksTable.dpcs [index].tskId, &pk_rtsk);
            if (erStatus == E_OK) {
                if (pk_rtsk.tskstat != TTS_DMT) {
                    if (pk_rtsk.actcnt > 0) {
                        erStatus = can_act (
                                         DPC_DpcTasksTable.dpcs [index].tskId) ;
                        if (erStatus == E_NOEXS || erStatus == E_ID) {
                            status = DSP_EFAIL ;
                            SET_FAILURE_REASON ;
                        }
                    }

                    if (DSP_SUCCEEDED (status)) {
                        erStatus = ter_tsk (
                                         DPC_DpcTasksTable.dpcs [index].tskId) ;
                        if (erStatus != E_OK) {
                            status = DSP_EFAIL ;
                            SET_FAILURE_REASON ;
                        }
                    }
                }
            }
            else {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }

            if (DSP_SUCCEEDED (status)) {
                erStatus = del_tsk (DPC_DpcTasksTable.dpcs [index].tskId) ;
                if (erStatus != E_OK) {
                    status = DSP_EFAIL ;
                    SET_FAILURE_REASON ;
                }
                else {
                    erId = acre_tsk(& DPC_DpcTasksTable.cTasks [index]);
                    if (erId > 0) {
                        DPC_DpcTasksTable.dpcs [index].tskId = erId ;
                    }
                    else {
                        status = DSP_EFAIL ;
                        SET_FAILURE_REASON ;
                    }
                }
            }
            SYNC_SpinLockEnd (irqFlags) ;
        }
        else {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("DPC_Cancel", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Schedule
 *
 *  @desc   This function schedules the DPC object, causing the user defined
 *          function associated with it to be invoked at a later point of time.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Schedule (IN  DpcObject * dpcObj)
{
    DSP_STATUS  status   = DSP_SOK    ;
    ER          erStatus = E_OK       ;
    Uint32      index    = 0          ;
    BOOL        state                 ;

    TRC_1ENTER ("DPC_Schedule", dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (dpcObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (dpcObj, SIGN_DPC)) ;
    DBC_Require ((dpcObj != NULL) && (dpcObj->index != INVALID_INDEX)) ;

    if (IS_OBJECT_VALID (dpcObj, SIGN_DPC) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        index = dpcObj->index ;
        if (index != INVALID_INDEX) {
            dpcObj->actCount++ ;
            if (DPC_IsEnabled == TRUE) {
                if (dpcObj->actCount <= MAX_DPC_ACTCOUNT) {
                    /* if activation count is more than one, i.e. task is ready to
                     * run by previous calls.
                     */
                    /** Each call to act_tsk will increment activation reference
                     *  count (maintained by PrOS).
                     */
                    /* get the current context. */
                    state = sns_ctx () ;

                    if (state == FALSE) {
                        /* It is a task context. */
                        erStatus = act_tsk (dpcObj->tskId) ;
                    }
                    else {
                        /* It is a interrupt context. */
                        erStatus = iact_tsk (dpcObj->tskId) ;
                    }

                    if (erStatus != E_OK) {
                        status = DSP_EFAIL ;
                        SET_FAILURE_REASON ;
                    }
                }
                else {
                    status = DSP_ERESOURCE ;
                    SET_FAILURE_REASON ;
                }
            }
        }
        else {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("DPC_Schedule", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Disable
 *
 *  @desc   Disables execution of DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
DPC_Disable ()
{

    TRC_0ENTER ("DPC_Disable") ;

    DPC_IsEnabled = FALSE;

    TRC_0LEAVE ("DPC_Disable") ;
}


/** ============================================================================
 *  @func   DPC_Enable
 *
 *  @desc   Enables execution of DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
DPC_Enable ()
{

    Uint32      i        = 0       ;

    TRC_0ENTER ("DPC_Enable") ;

    DPC_IsEnabled = TRUE ;

    for (i = 0 ; i < MAX_DPC ; i++) {
        /* release the lock, so that DPC takes it and run */
        if (DPC_DpcTasksTable.dpcs [i].actCount > 0) {
            DPC_Schedule (&(DPC_DpcTasksTable.dpcs [i]));
        }
    }

    TRC_0LEAVE ("DPC_Enable") ;
}

/** ----------------------------------------------------------------------------
 *  @func   DPC_Callback
 *
 *  @desc   This function gets invoked when an DPC is invoked.
 *          Since ext_tsk is matched with act_tsk by using an activation
 *          reference count (maintained by PrOS). So, we simply keep calling
 *          ext_tsk in a loop, when reference count becomes zero, this task
 *          automatically becomes dormant.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
void
DPC_Callback (VP_INT exinf)
{
    DpcObject * dpcObj   = NULL    ;
#if defined (DDSP_DEBUG)
    DSP_STATUS  status   = DSP_SOK ;
#endif /* if defined (DDSP_DEBUG) */
    Uint32     irqFlags           ;

    TRC_1ENTER ("DPC_Callback", exinf) ;

    DBC_Require ((Uint16) exinf < MAX_DPC) ;

    dpcObj = &DPC_DpcTasksTable.dpcs [(Uint16) exinf] ;

    if (IS_OBJECT_VALID (dpcObj, SIGN_DPC)) {
        if (DPC_IsEnabled == TRUE) {
            while (dpcObj->actCount != 0) {
                /** If the activation reference count becomes zero, call to ext_tsk
                 *  will terminate this task i.e, DPC_Callback will exit and
                 *  loop will terminate.
                 */
                irqFlags = SYNC_SpinLockStart () ;
                dpcObj->actCount-- ;
                SYNC_SpinLockEnd (irqFlags) ;

                if (*dpcObj->userDPCFn != NULL) {
                    (*dpcObj->userDPCFn) (dpcObj->paramData) ;
                }
#if defined (DDSP_DEBUG)
                else {
                    status = DSP_EPOINTER ;
                    SET_FAILURE_REASON ;
                }
#endif /* if defined (DDSP_DEBUG) */
            }
        }
    }

    ext_tsk () ;

    TRC_0LEAVE ("DPC_Callback") ;
}


#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @func   DPC_Debug
 *
 *  @desc   This function prints the current status of DPC objects in
 *          the 'Link.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
DPC_Debug ()
{
    Uint32 i = 0 ;

    TRC_0ENTER ("DPC_Debug") ;

    TRC_1PRINT (TRC_LEVEL1, "DPC_DpcTasksTable.usedDPCs: [0x%x]\n",
                            DPC_DpcTasksTable.usedDPCs) ;
    for (i = 0 ; i < MAX_DPC ; i++) {
        TRC_1PRINT (TRC_LEVEL1, "DPC_DpcTasksTable.dpcs [0x%x]\n", i) ;
        TRC_1PRINT (TRC_LEVEL1, "   signature: [0x%x]\n",
                                DPC_DpcTasksTable.dpcs[i].signature) ;
        TRC_1PRINT (TRC_LEVEL1, "   userDPCFn: [0x%x]\n",
                                DPC_DpcTasksTable.dpcs[i].userDPCFn) ;
        TRC_1PRINT (TRC_LEVEL1, "   paramData: [0x%x]\n",
                                DPC_DpcTasksTable.dpcs[i].paramData) ;
        TRC_1PRINT (TRC_LEVEL1, "       tskId: [0x%x]\n",
                                DPC_DpcTasksTable.dpcs[i].tskId) ;
        TRC_1PRINT (TRC_LEVEL1, "       index: [0x%x]\n",
                                DPC_DpcTasksTable.dpcs[i].index) ;
    }

    TRC_0LEAVE ("DPC_Debug") ;
}
#endif /* defined (DDSP_DEBUG) */


#if defined (__cplusplus)
}
#endif
