/** ============================================================================
 *  @file   isr.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implementation of sub-component ISR. In PrOS only one interrupt
 *          handler can be registered for single interrupt. Also, now two ISRs
 *          are supported (physical link + probe).
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

/*  ----------------------------------- OS Specific Headers         */
#include <itron.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_ISR

/** ============================================================================
 *  @const  MAX_ISR
 *
 *  @desc   Maximum number of ISRs supported.
 *  ============================================================================
 */
#define MAX_ISR             256

/** ============================================================================
 *  @const  BH_PRIORITY
 *
 *  @desc   Priority value of bottom half tasks.
 *  ============================================================================
 */
#define BH_PRIORITY     3u

/** ============================================================================
 *  @const  BH_STACK_SIZE
 *
 *  @desc   Stack size of Bottom Half.
 *  ============================================================================
 */
#define BH_STACK_SIZE  1024u


/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_ISR, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG)  */


/** ============================================================================
 *  @name   IsrObject_tag
 *
 *  @desc   Defines object to encapsulate the interrupt service routine.
 *          The definition is Only OS-specific.
 *
 *  @field  signature
 *              Signature to identify this object.
 *  @field  refData
 *              Data to be passed to ISR on invocation.
 *  @field  fnISR
 *              Actual Interrupt service routine.
 *  @field  irq
 *              The IRQ number.
 *  @field  installed
 *              Flag to indicate the ISR is installed.
 *  @field  enabled
 *              Flag to indicate the ISR is enabled.
 *  @field  tskId
 *              Task Id for Bottom Half task.
 *  @field  bhStkPtr
 *              Stack pointer for associated BH task.
 *  @field  checkFunc
 *              Function to check and  clear interrupt.
 *  @field  param
 *             void pointer passed to checkFunc(i.e ipsState).
 *  ============================================================================
 */

struct IsrObject_tag {
    Uint32       signature ;
    Pvoid        refData   ;
    IsrProc      fnISR     ;
    INT          irq       ;
    Bool         installed ;
    Bool         enabled   ;
    Uint32       tskId     ;
    Pvoid        bhStkPtr  ;
    INT_checkGen checkFunc ;
    Pvoid        param     ;
} ;



/** ----------------------------------------------------------------------------
 *  @func   ISR_Callback
 *
 *  @desc   This function is registered as an interrupt handler for all the
 *          irqs on which user wants to register an ISR.
 *          The Linux kernel calls this function when an interrupt occurs on
 *          the specified irq. This function, then, looks up the irq and
 *          invokes the user specified interrupt service routine.
 *
 *  @arg    irq
 *              vector on which the interrupt occurred
 *  @arg    opt
 *              optional argument.
 *
 *  @ret    None.
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    ISR_Install
 *  ----------------------------------------------------------------------------
 */
STATIC
void
ISR_Callback (int irq, int opt) ;

/*  ============================================================================
 *  @name   ISR_InstalledIsrs
 *
 *  @desc   Array to hold all the installed  isrObjects.
 *  ============================================================================
 */
STATIC IsrObject * ISR_InstalledIsrs [MAX_ISR] ;

/*  ============================================================================
 *  @name   ISR_IsInitialized
 *
 *  @desc   Flag for tracking initialization of subcomponent.
 *  ============================================================================
 */
STATIC Bool  ISR_IsInitialized = FALSE ;


/** ----------------------------------------------------------------------------
 *  @func   ISR_BH
 *
 *  @desc   Entry point for all Bottom Halves.
 *
 *  @arg    None
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ----------------------------------------------------------------------------
 */
void
ISR_BH (VP_INT exinf) ;


/** ============================================================================
 *  @func   ISR_Initialize
 *
 *  @desc   Initialize the module and allocate resource used by this module.
 *
 *  @modif  ISR_InstalledIsr1, ISR_InstalledIsr2, ISR_IsInitialized
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;
    Uint32      index  = 0       ;

    TRC_0ENTER ("Entered ISR_Initialize ()\n") ;

    DBC_Require (ISR_IsInitialized == FALSE) ;

    if (ISR_IsInitialized == FALSE) {
        for (index = 0 ; index < MAX_ISR ; index ++) {
            ISR_InstalledIsrs [index] = NULL ;
        }
        ISR_IsInitialized = TRUE ;
    }

    DBC_Ensure (ISR_IsInitialized == TRUE) ;

    TRC_1LEAVE ("ISR_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Finalize
 *
 *  @desc   This function provides an interface to exit from the ISR module.
 *          It frees up all the used ISRs and releases all the resources used by
 *          this module.
 *
 *  @modif  ISR_InstalledIsr1, ISR_InstalledIsr2, ISR_IsInitialized
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;
    DSP_STATUS tmpStatus = DSP_SOK ;
    Uint32      index  = 0       ;

    TRC_0ENTER ("ISR_Finalize") ;

    DBC_Require (ISR_IsInitialized == TRUE) ;

    if (ISR_IsInitialized == TRUE) {
        for (index  = 0 ; index < MAX_ISR ; index ++) {
            if (ISR_InstalledIsrs [index] != NULL) {
                tmpStatus = ISR_Uninstall (ISR_InstalledIsrs [index]) ;
                if (DSP_FAILED (tmpStatus) && DSP_SUCCEEDED (status)) {
                    status = tmpStatus ;
                }
            }
        }
        ISR_IsInitialized = FALSE ;
    }

    DBC_Ensure (ISR_IsInitialized == FALSE) ;

    TRC_1LEAVE ("ISR_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Create
 *
 *  @desc   Creates an ISR object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Create (IN  IsrProc           fnISR,
            IN  Pvoid             refData,
            IN  InterruptObject * intObj,
            OUT IsrObject **      isrObj)
{
    DSP_STATUS   status   = DSP_SOK ;
    ER_ID        erId        = E_OK    ;
    T_CTSK       ctsk  ;

    TRC_4ENTER ("ISR_Create", fnISR, refData, intObj, isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (fnISR != NULL)  ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (intObj != NULL) ;
    DBC_Require ((intObj != NULL) && (intObj->intId >= 0)) ;

    if (   (isrObj == NULL)
        || (fnISR == NULL)
        || (intObj == NULL)
        || (intObj->intId < 0)) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status = MEM_Alloc ((Void **) isrObj, sizeof (IsrObject), MEM_DEFAULT) ;

        if (DSP_SUCCEEDED (status)) {
            (*isrObj)->signature = SIGN_ISR       ;
            (*isrObj)->fnISR     = fnISR          ;
            (*isrObj)->irq       = intObj->intId  ;
            (*isrObj)->installed = FALSE          ;
            (*isrObj)->refData   = refData        ;
            (*isrObj)->enabled   = FALSE          ;
            (*isrObj)->checkFunc = intObj->checkFunc  ;
            (*isrObj)->param     = intObj->param  ;

            ctsk.exinf   = (VP_INT) (Pvoid) (*isrObj) ;
            ctsk.tskatr  = TA_HLNG           ;
            ctsk.task    = (FP) ISR_BH       ;
            ctsk.itskpri = BH_PRIORITY       ;
            ctsk.stksz   = BH_STACK_SIZE     ;

            /* Allocate memory for stack */
            status = MEM_Alloc ((Pvoid*) &ctsk.stk,
                                ctsk.stksz,
                                MEM_DEFAULT);
            if (DSP_FAILED (status)) {
                (*isrObj)->signature = SIGN_NULL ;
                FREE_PTR (isrObj) ;
                SET_FAILURE_REASON ;
            }
            else {
                (*isrObj)->bhStkPtr = ctsk.stk ;
                erId = acre_tsk(&ctsk);
                if (erId > 0) {
                    (*isrObj)->tskId = erId ;
                }
                else {
                    FREE_PTR((*isrObj)->bhStkPtr) ;
                    FREE_PTR (isrObj) ;
                    status = DSP_EFAIL ;
                    SET_FAILURE_REASON ;
                }
            }
        }
        else {
            SET_FAILURE_REASON ;
        }
    }

    DBC_Ensure (  (   DSP_SUCCEEDED (status)
                   && IS_OBJECT_VALID (*isrObj, SIGN_ISR))
                || DSP_FAILED (status)) ;

    TRC_1LEAVE ("ISR_Create", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Delete
 *
 *  @desc   Delete the isrObject.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Delete (IN IsrObject * isrObj)
{
    DSP_STATUS status = DSP_SOK ;
    DSP_STATUS tmpStatus1 = DSP_SOK ;
    DSP_STATUS tmpStatus2 = DSP_SOK ;
    ER         erStatus   = E_OK    ;
    T_RTSK     pk_rtsk              ;

    TRC_1ENTER ("ISR_Delete", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        /*  --------------------------------------------------------------------
         *  Verify that the ISR is not installed before deleting the object
         *  --------------------------------------------------------------------
         */
        if (isrObj->installed == FALSE) {
            /* Get the process info */
            erStatus = ref_tsk (isrObj->tskId, &pk_rtsk);
            if (erStatus == E_OK) {
                if (pk_rtsk.tskstat != TTS_DMT) {
                    /* Change the status of Bottom Half */
                    erStatus = ter_tsk (isrObj->tskId) ;
                    if (erStatus != E_OK) {
                        status = DSP_EFAIL ;
                        SET_FAILURE_REASON ;
                    }
                }

                if (DSP_SUCCEEDED (status)) {
                    erStatus = del_tsk (isrObj->tskId) ;
                    if (erStatus != E_OK) {
                        status = DSP_EFAIL ;
                        SET_FAILURE_REASON ;
                    }
                }
            }
            else {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }


            isrObj->signature = SIGN_NULL ;
            tmpStatus1 = FREE_PTR (isrObj->bhStkPtr) ;
            tmpStatus2 = FREE_PTR (isrObj) ;
            if (   (DSP_FAILED (tmpStatus1) || DSP_FAILED (tmpStatus2))
                && (DSP_SUCCEEDED (status))) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
        }
        else {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("ISR_Delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Install
 *
 *  @desc   Install an interrupt service routine.
 *
 *  @modif  ISR_InstalledIsr1, ISR_InstalledIsr2
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Install (IN  Void *      hostConfig,
             IN  IsrObject * isrObj)
{
    DSP_STATUS    status      = DSP_SOK ;
    ER            erStatus    = E_OK ;
    T_DINH        pkDinh ;

    TRC_2ENTER ("ISR_Install", hostConfig, isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj != NULL)
                 && (ISR_InstalledIsrs [isrObj->irq] == NULL)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->irq] != NULL) {
        status = DSP_EACCESSDENIED ;
        SET_FAILURE_REASON ;
    }
    else {
        if (DSP_SUCCEEDED (status)) {
            pkDinh.inhatr = 0 ;
            pkDinh.inthdr = (FP) ISR_Callback ;
            erStatus = def_inh (isrObj->irq, &pkDinh) ;

            if (erStatus != E_OK) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else {
                isrObj->installed = TRUE ;
                ISR_InstalledIsrs [isrObj->irq] = isrObj ;
                /* Enable the interrupt */
                status = ISR_Enable (isrObj) ;
                if (DSP_SUCCEEDED (status)) {
                    isrObj->enabled = TRUE  ;
                }
                else {
                    SET_FAILURE_REASON ;
                }
            }
        }
    }

    DBC_Ensure (   (   (DSP_SUCCEEDED (status))
                    && (ISR_InstalledIsrs [isrObj->irq] == isrObj))
                || DSP_FAILED (status)) ;

    TRC_1LEAVE ("ISR_Install", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Uninstall
 *
 *  @desc   Uninstalls an ISR.
 *
 *  @modif  ISR_InstalledIsr1, ISR_InstalledIsr2
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Uninstall (IN IsrObject * isrObj)
{
    DSP_STATUS  status      = DSP_SOK ;
    ER          erStatus    = E_OK    ;

    TRC_1ENTER ("ISR_Uninstall", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj!= NULL)
                 && (ISR_InstalledIsrs [isrObj->irq] == isrObj)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->irq] != isrObj) {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
    }
    else {
        if (isrObj->enabled == TRUE) {
            status = ISR_Disable (isrObj) ;
        }

        if (DSP_SUCCEEDED (status)) {
            erStatus = def_inh (isrObj->irq, (T_DINH *) NULL) ;

            if (erStatus != E_OK) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else {
                ISR_InstalledIsrs [isrObj->irq] = NULL ;
                isrObj->installed = FALSE ;
            }
        }
    }

    TRC_1LEAVE ("ISR_Uninstall", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Disable
 *
 *  @desc   Disables the interrupt handler.
 *
 *  @modif  none
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Disable (IN  IsrObject * isrObj)
{
    DSP_STATUS status    = DSP_SOK ;
    ER         erStatus  = E_OK    ;

    TRC_1ENTER ("ISR_Disable", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj != NULL)
                 && (ISR_InstalledIsrs [isrObj->irq] == isrObj)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->irq] != isrObj) {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
    }
    else {
        erStatus = dis_int (isrObj->irq) ;
        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
        else {
            isrObj->enabled = FALSE ;
        }
    }

    TRC_1LEAVE ("ISR_Disable", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Enable
 *
 *  @desc   Enables the interrupt handler.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Enable (IN  IsrObject * isrObj)
{
    DSP_STATUS status    = DSP_SOK ;
    ER         erStatus  = E_OK    ;

    TRC_1ENTER ("ISR_Enable", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj != NULL)
                 && (ISR_InstalledIsrs [isrObj->irq] == isrObj)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->irq] != isrObj) {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
        }
    else {
        erStatus = ena_int (isrObj->irq) ;
        if (erStatus == E_OK) {
            isrObj->enabled = TRUE ;
        }
        else {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("ISR_Enable", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_GetState
 *
 *  @desc   Gets the state of an ISR.
 *          This function first checks if the ISR was installed using this
 *          subcomponent. If yes, then it checks the enabled flag of isrObject.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_GetState (IN  IsrObject *  isrObj,
              OUT ISR_State *  isrState)
{
    DSP_STATUS  status      = DSP_SOK ;

    TRC_2ENTER ("ISR_GetState", isrObj, isrState) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (isrState != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE)  {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (isrState == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }

    if (DSP_SUCCEEDED (status)) {
        *isrState = ISR_Uninstalled ;

            if (ISR_InstalledIsrs [isrObj->irq] == isrObj) {
                if (isrObj->enabled == TRUE) {
                    *isrState = ISR_Enabled ;
                }
                else {
                    *isrState = ISR_Disabled ;
                }
            }
        }

    TRC_1LEAVE ("ISR_GetState", status) ;

    return status ;
}


#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @func   ISR_Debug
 *
 *  @desc   Prints the current status of ISR objects in the system.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
ISR_Debug ()
{
    Int32  i = 0 ;

    TRC_0ENTER ("ISR_Debug ()\n") ;

    DBC_Require (ISR_IsInitialized == TRUE) ;

    for (i = 0 ; i < MAX_ISR ; i++) {
        if (ISR_InstalledIsrs [i] != NULL) {
            TRC_1PRINT (TRC_LEVEL1, "ISR_InstalledIsrs [0x%x]\n", i) ;
            TRC_1PRINT (TRC_LEVEL1, "   signature:   [0x%x]\n",
                                    ISR_InstalledIsrs [i]->signature) ;
            TRC_1PRINT (TRC_LEVEL1, "   refData:     [0x%x]\n",
                                    ISR_InstalledIsrs [i]->refData) ;
            TRC_1PRINT (TRC_LEVEL1, "   fnISR:       [0x%x]\n",
                                    ISR_InstalledIsrs [i]->fnISR) ;
            TRC_1PRINT (TRC_LEVEL1, "   interruptId: [0x%x]\n",
                                    ISR_InstalledIsrs [i]->irq) ;
            TRC_1PRINT (TRC_LEVEL1, "   enabled:    [0x%x]\n",
                                    ISR_InstalledIsrs [i]->enabled) ;
        }
    }


    TRC_0PRINT (TRC_ENTER, "Leaving ISR_Debug ()\n") ;
}
#endif /* defined (DDSP_DEBUG) */


/** ----------------------------------------------------------------------------
 *  @func   ISR_Callback
 *
 *  @desc   This function is registered as an interrupt handler for the first
 *          vector on which user wants to register the ISR. On occurrence of
 *          the interrupt this function is invoked and it calls the user defined
 *          ISR with appropriate parameters.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
STATIC
void
ISR_Callback (int intNo, int opt)
{
#if defined (DDSP_DEBUG)
    DSP_STATUS status = DSP_SOK ;
#endif /* if defined (DDSP_DEBUG) */
    IsrObject *  isrObj     = (IsrObject *) (ISR_InstalledIsrs[intNo]) ;
    Bool         isAsserted = FALSE ;

    DBC_Assert (ISR_InstalledIsrs[intNo] != NULL) ;

    ISR_Disable (ISR_InstalledIsrs[intNo]) ;

    if ((isrObj != NULL) &&
        IS_OBJECT_VALID (ISR_InstalledIsrs[intNo], SIGN_ISR) == TRUE) {
        if (isrObj->checkFunc) {
            isAsserted = (*isrObj->checkFunc) (isrObj->param) ;
            if (isAsserted == TRUE) {
                (*(ISR_InstalledIsrs[intNo]->fnISR))
                                           (ISR_InstalledIsrs[intNo]->refData) ;
            }
        }
    }
#if defined (DDSP_DEBUG)
    else {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
#endif /* if defined (DDSP_DEBUG) */

    iact_tsk (ISR_InstalledIsrs[intNo]->tskId) ;
}


/** ----------------------------------------------------------------------------
 *  @func   ISR_BH
 *
 *  @desc   Entry point for all bottom halves.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
void
ISR_BH (VP_INT exinf)
{
#if defined (DDSP_DEBUG)
    DSP_STATUS status = DSP_SOK ;
#endif /* if defined (DDSP_DEBUG) */
    IsrObject * isrObj  = (IsrObject *) exinf ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == TRUE) {
        ISR_Enable (isrObj) ;
    }
#if defined (DDSP_DEBUG)
    else {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
#endif /* if defined (DDSP_DEBUG) */
}

#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
