/** ============================================================================
 *  @file   mem.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implementation of sub-component MEM.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Headers                  */
#include <itron.h>
#include <string.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_OSAL_MEM

/** ============================================================================
 *  @macro  MPL_BUFF_SIZE
 *
 *  @desc   Memory pool size.
 *  ============================================================================
 */
#define  MPL_BUFF_SIZE  0x200000u

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_OSAL_MEM, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG)  */


/** ============================================================================
 *  @name   MEM_IsInitialized
 *
 *  @desc   Flag to track initialization of the sub-component.
 *  ============================================================================
 */
STATIC Bool MEM_IsInitialized  = FALSE ;

/** ============================================================================
 *  @name   MEM_mplId
 *
 *  @desc   ID of memory pool created.
 *  ============================================================================
 */
STATIC ID MEM_mplId ;


/** ============================================================================
 *  @name   mplbuff
 *
 *  @desc   Start address of the variable-sized memory pool area.
 *  ============================================================================
 */
Uint8 mplbuff [MPL_BUFF_SIZE] ;


/** ============================================================================
 *  @func   MEM_Initialize
 *
 *  @desc   Initializes the MEM sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Initialize ()
{
    DSP_STATUS status   = DSP_SOK ;
    T_CMPL     pk_cmpl            ;
    ER_ID      mplId              ;

    TRC_0ENTER ("MEM_Initialize") ;

    DBC_Require (MEM_IsInitialized == FALSE) ;

    if (MEM_IsInitialized == FALSE) {
        pk_cmpl.mplatr = TA_TPRI ;
        pk_cmpl.mplsz = MPL_BUFF_SIZE ;
        pk_cmpl.mpl = (VP) mplbuff ;
        mplId = acre_mpl (&pk_cmpl) ;

        if (mplId > 0) {
            MEM_mplId = mplId ;
            MEM_IsInitialized = TRUE ;
        }
        else {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("MEM_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Finalize
 *
 *  @desc   Releases resources used by this sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Finalize ()
{
    DSP_STATUS status   = DSP_SOK ;
    ER_ID      erStatus           ;

    TRC_0ENTER ("MEM_Finalize") ;

    DBC_Require (MEM_IsInitialized == TRUE) ;

    if (MEM_IsInitialized == TRUE) {
        erStatus = del_mpl (MEM_mplId) ;

        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
        else {
            MEM_IsInitialized = FALSE ;
            MEM_mplId = -1 ;
        }
    }

    TRC_1LEAVE ("MEM_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Alloc
 *
 *  @desc   Allocates the specified number of bytes.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Alloc (OUT Void ** ptr, IN Uint32 cBytes, IN OUT Pvoid arg)
{
    DSP_STATUS   status   = DSP_SOK    ;
    ER           erStatus = E_OK       ;
    MemAllocAttrs * memAllocAttrs = NULL  ;

    TRC_3ENTER ("MEM_Alloc", ptr, cBytes, arg) ;

    DBC_Require (ptr != NULL) ;
    DBC_Require (cBytes > 0) ;
    DBC_Require (MEM_IsInitialized == TRUE) ;

    /*  ------------------------------------------------------------------------
     *  Validate arguments
     *  ------------------------------------------------------------------------
     */
    if (ptr == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else if (cBytes == 0) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;

        *ptr = NULL ;
    }
    else {
        /*  --------------------------------------------------------------------
         *  Allocations from 'default' memory area
         *  --------------------------------------------------------------------
         */
        erStatus = get_mpl (MEM_mplId, (UINT) cBytes, (VP *) ptr) ;

        if (erStatus != E_OK) {
            status = DSP_EMEMORY ;
            SET_FAILURE_REASON ;
        }
        else {
            if (arg != MEM_DEFAULT) {
                memAllocAttrs = (MemAllocAttrs *) arg ;
                memAllocAttrs->physicalAddress = (Uint32 *) *ptr ;
            }
        }
    }

    TRC_1LEAVE ("MEM_Alloc", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Calloc
 *
 *  @desc   Allocates the specified number of bytes and memory is set to zero.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Calloc (OUT Void ** ptr, IN Uint32 cBytes, IN OUT Pvoid arg)
{
    DSP_STATUS   status = DSP_SOK ;
    Uint32       i                ;
    Uint8 *      temp   = NULL    ;

    TRC_3ENTER ("MEM_Calloc", ptr, cBytes, arg) ;

    DBC_Require (ptr != NULL) ;
    DBC_Require (cBytes > 0) ;
    DBC_Require (MEM_IsInitialized == TRUE) ;

    status = MEM_Alloc (ptr, cBytes, arg) ;

    if (DSP_SUCCEEDED (status)) {
        temp = *((Uint8 **)ptr) ;
        for (i = 0 ; i < cBytes ; i++) {
            *temp++ = 0 ;
        }
    }

    TRC_1LEAVE ("MEM_Calloc", status) ;

    return  status ;
}


/** ============================================================================
 *  @func   MEM_Free
 *
 *  @desc   Frees up the specified chunk of memory.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Free (IN Pvoid * ptr, IN Pvoid arg)
{
    DSP_STATUS status   = DSP_SOK    ;
    ER         erStatus = E_OK       ;

    TRC_2ENTER ("MEM_Free", ptr, arg) ;

    DBC_Require (ptr != NULL) ;
    DBC_Require (MEM_IsInitialized == TRUE) ;

    /*  ------------------------------------------------------------------------
     *  Validate arguments
     *  ------------------------------------------------------------------------
     */
    if (ptr == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (*ptr == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        /*  ----------------------------------------------------------------
         *  Free allocations from 'default' memory area
         *  ----------------------------------------------------------------
         */
        erStatus = rel_mpl (MEM_mplId, (VP) *ptr) ;

        if (erStatus != E_OK) {
            status = DSP_EFREE ;
            SET_FAILURE_REASON ;
        }
        else {
            *ptr = NULL ;
        }
    }

    TRC_1LEAVE ("MEM_Free", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Map
 *
 *  @desc   Maps a memory area into virtual space. In Nucleus no actual Memory
 *          mapping is happening.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Map (IN OUT MemMapInfo * mapInfo)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("MEM_Map", mapInfo) ;

    DBC_Require (mapInfo != NULL) ;

    if (mapInfo != NULL) {
        mapInfo->dst = mapInfo->src ;
    }

    TRC_1LEAVE ("MEM_Map", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Unmap
 *
 *  @desc   Unmaps a virtual memory area. In PrOS no actual Memory
 *          unmapping is happening.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Unmap (IN MemUnmapInfo * unmapInfo)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("MEM_Unmap", unmapInfo) ;

    DBC_Require (unmapInfo != NULL) ;

    TRC_1LEAVE ("MEM_Unmap", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Copy
 *
 *  @desc   Copies the data between memory areas.
 *          (The endianism is currently not used.)
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Copy (IN Uint8 * dst, OUT Uint8 * src, IN Uint32 len, Endianism endianism)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_4ENTER ("MEM_Copy", dst, src, len, endianism) ;

    memcpy (dst, src, len) ;

    TRC_1LEAVE ("MEM_Copy", status) ;

    return status ;
}


#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @func   MEM_Debug
 *
 *  @desc   Prints debug information for MEM.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
MEM_Debug ()
{
    ER      erStatus ;
    T_RMPL  pk_rmpl ;

    TRC_0ENTER ("MEM_Debug") ;

    erStatus = ref_mpl (MEM_mplId, &pk_rmpl) ;
    if (erStatus == E_OK) {
        TRC_3PRINT (TRC_LEVEL4,
                "    ID number of the task at the head of the wait queue [%d]\n"
                "    Total size of free memory blocks (in bytes)  [%d]\n"
                "    Maximum memory block size available (in bytes) [%d]\n",
                 pk_rmpl.wtskid,
                 pk_rmpl.fmplsz,
                 pk_rmpl.fblksz) ;
    }

    TRC_0LEAVE ("MEM_Debug") ;
}
#endif /* defined (DDSP_DEBUG) */


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
