/** ============================================================================
 *  @file   isr.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/WinCE/
 *
 *  @desc   Implementation of sub-component ISR.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>
#include <pkfuncs.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_ISR

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_ISR, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */

/** ============================================================================
 *  @const  MAX_ISR
 *
 *  @desc   Maximum number of ISRs supported.
 *  ============================================================================
 */
#define MAX_ISR             256

/** ============================================================================
 *  @const  ISR_THREAD_PRIORITY
 *
 *  @desc   Priority at which the IST thread runs
 *  ============================================================================
 */

#define ISR_THREAD_PRIORITY 90




/** ============================================================================
 *  @name   IsrObject_tag
 *
 *  @desc   Defines object to encapsulate the interrupt service routine.
 *          The definition is OS/platform specific.
 *
 *  @field  signature
 *              Signature to identify this object.
 *  @field  dspId
 *              Processor Identifier.
 *  @field  irq
 *              The IRQ number.
 *  @field  enabled
 *              Flag to indicate the ISR is enabled. 
 *  @field  checkFunc
 *              Function to check for interrupt generating device.
 *  @field  param
 *              Parameter to be passed to check function.
 *  @field  hIsrEvent
 *              Event handle for the ISR
 *  @field  hIsrThread
 *              Thread handle for the IST
 *  @field  dwSysIntr
 *              Interrupt identifier
 *  @field  fnISR
 *              Actual Interrupt service routine.
 *  @field  refData
 *              Data to be passed to ISR on invocation.
 *  
 *  ============================================================================
 */
struct IsrObject_tag {
    Uint32       signature ;
    ProcessorId  dspId     ;
    int          irq       ;
    Bool         enabled   ;
    INT_checkGen checkFunc ;
    Pvoid        param     ;
    HANDLE       hIsrEvent ;
    HANDLE       hIsrThread;
    DWORD        dwSysIntr ;
    IsrProc      isrProcfn ;
    Pvoid        refdata   ;
} ;


/** ----------------------------------------------------------------------------
 *  @func   ISR_Callback
 *
 *  @desc   This function is the entry point for the IST created for the interrupt 
 *          on which user wants to install an ISR.
 *          This function waits for the event that is generated an interrupt occurs on
 *          the specified irq. This function then invokes the user specified interrupt 
 *          service routine.
 *
 *  @arg    arg
 *              Device object - contains the user defined object that was used
 *              when handler was registered.
 *
 *  @ret    status
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    ISR_Install
 *  ----------------------------------------------------------------------------
 */
DWORD
ISR_Callback (LPVOID arg) ;


/*  ============================================================================
 *  @name   ISR_IsInitialized
 *
 *  @desc   Flag for tracking initialization of subcomponent.
 *  ============================================================================
 */
STATIC Bool        ISR_IsInitialized = FALSE ;

/*  ============================================================================
 *  @name   ISR_InstalledIsrs
 *
 *  @desc   Array to hold all the installed  isrObjects.
 *  ============================================================================
 */
STATIC IsrObject * ISR_InstalledIsrs [MAX_DSPS][MAX_ISR] ;


/** ============================================================================
 *  @func   ISR_Initialize
 *
 *  @desc   Initialize the module and allocate resource used by this module.
 *
 *  @modif  ISR_InstalledIsrs, ISR_IsInitialized
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;
    Uint32      index  = 0       ;
    Uint32      procId = 0       ;

    TRC_0ENTER ("Entered ISR_Initialize ()\n") ;

    DBC_Require (ISR_IsInitialized == FALSE) ;

    if (ISR_IsInitialized == FALSE) {
        for (procId = 0 ; procId < MAX_DSPS ; procId ++) {
            for (index = 0 ; index < MAX_ISR ; index ++) {
                ISR_InstalledIsrs [procId][index] = NULL ;
            }
        }
        ISR_IsInitialized = TRUE ;
    }

    DBC_Ensure (ISR_IsInitialized == TRUE) ;

    TRC_1LEAVE ("ISR_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Finalize
 *
 *  @desc   This function provides an interface to exit from the ISR module.
 *          It frees up all the used ISRs and releases all the resources used by
 *          this module.
 *
 *  @modif  ISR_InstalledIsrs, ISR_IsInitialized
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;
    DSP_STATUS tmpStatus = DSP_SOK ;
    Uint32      index  = 0       ;
    Uint32      procId = 0       ;

    TRC_0ENTER ("ISR_Finalize") ;

    DBC_Require (ISR_IsInitialized == TRUE) ;

    if (ISR_IsInitialized == TRUE) {
        for (procId = 0 ; procId < MAX_DSPS ; procId ++) {
            for (index = 0 ; index < MAX_ISR ; index ++) {
                if (ISR_InstalledIsrs [procId][index] != NULL) {
                    tmpStatus = ISR_Uninstall (
                                            ISR_InstalledIsrs [procId][index]) ;
                    if (DSP_FAILED (tmpStatus) && DSP_SUCCEEDED (status)) {
                        status = tmpStatus ;
                    }
                }
            }
        }
        ISR_IsInitialized = FALSE ;
    }

    DBC_Ensure (ISR_IsInitialized == FALSE) ;

    TRC_1LEAVE ("ISR_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Create
 *
 *  @desc   Creates an ISR object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Create (IN  IsrProc             fnISR,
            IN  Pvoid               refData,
            IN  InterruptObject *   intObj,
            OUT IsrObject **        isrObj)
{
    DSP_STATUS   status   = DSP_SOK ;

    TRC_4ENTER ("ISR_Create", fnISR, refData, intObj, isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (fnISR != NULL)  ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (intObj != NULL) ;
    DBC_Require (   (intObj != NULL)
                 && (intObj->intId >= 0)
                 && (intObj->intId < MAX_ISR)) ;

    if (   (fnISR  == NULL)
        || (intObj == NULL)
        || (isrObj == NULL)
        || (intObj->intId < 0)
        || (intObj->intId >= MAX_ISR)) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status = MEM_Alloc ((Void **) isrObj,
                            sizeof (IsrObject),
                            MEM_DEFAULT) ;

        if (DSP_SUCCEEDED (status)) {
            (*isrObj)->signature = SIGN_ISR       ;
            (*isrObj)->irq       = intObj->intId  ;
            (*isrObj)->enabled   = FALSE          ;
            (*isrObj)->dspId     = intObj->dspId  ;
            (*isrObj)->checkFunc = intObj->checkFunc  ;            
            (*isrObj)->param     = intObj->param  ;
            (*isrObj)->isrProcfn = fnISR          ;
            (*isrObj)->refdata   = refData        ;
            /* If shared is TRUE, checkFunc must be provided to check if the
             * interrupt is asserted.
             */
            if (intObj->shared == TRUE) {
                if (intObj->checkFunc == NULL) {
                    status = DSP_EINVALIDARG ;
                    SET_FAILURE_REASON ;
                }
            }            
        }
        else {
            SET_FAILURE_REASON ;
        }
    }


    DBC_Ensure (  (   DSP_SUCCEEDED (status)
                   && IS_OBJECT_VALID (*isrObj, SIGN_ISR))
                || DSP_FAILED (status)) ;

    TRC_1LEAVE ("ISR_Create", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Delete
 *
 *  @desc   Delete the isrObject.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Delete (IN IsrObject * isrObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("ISR_Delete", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE)          ;
    DBC_Require (isrObj != NULL)                     ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        /*  --------------------------------------------------------------------
         *  Verify that the ISR is not installed before deleting the object
         *  --------------------------------------------------------------------
         */
        if (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] != isrObj) {            
            isrObj->signature = SIGN_NULL ;
            status = FREE_PTR (isrObj) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
        else {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("ISR_Delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Install
 *
 *  @desc   Install an interrupt service routine.
 *          This function creates Interrupt Identifier and an event handler, 
 *          then associates them with each other. It then creates an IST that 
 *          will wait on the event (that gets signalled when the respective 
 *          interrupt occurs)
 *
 *  @modif  ISR_InstalledIsrs
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Install (IN  Void *      hostConfig,
             IN  IsrObject * isrObj)
{
    DSP_STATUS   status   = DSP_SOK ;

    DSP_STATUS   tmpstatus   = DSP_SOK ;
    int          osStatus = 0       ;
    DWORD        dwSysIntrVal = 0;    

    Uint32       threadPRI = 0;
    Uint32       bfrSz=0;


    TRC_2ENTER ("ISR_Install", hostConfig, isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj != NULL)
                 && (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] == NULL)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] != NULL) {
        status = DSP_EACCESSDENIED ;
        SET_FAILURE_REASON ;
    }
    else {
        /* get the SysIntr value */        
        osStatus = KernelIoControl(IOCTL_HAL_REQUEST_SYSINTR, 
                                  &(isrObj->irq), 
                                  sizeof(DWORD), &dwSysIntrVal, 
                                  sizeof(DWORD), NULL);
        if (osStatus == FALSE)
        {   
            TRC_1PRINT (TRC_LEVEL7, "KernelIoControl for IOCTL_HAL_REQUEST_SYSINTR failed with error: %d\n",
                        osStatus) ;
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON;
        }

        if (DSP_SUCCEEDED(status))
        {   
            isrObj->dwSysIntr = dwSysIntrVal;
            
            
            /* create an event */            
            isrObj->hIsrEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
            if (isrObj->hIsrEvent == NULL)
            {            
                TRC_0PRINT (TRC_LEVEL7, "CreateEvent failed :\n");
                status = DSP_EFAIL;
                /* release sysIntr */
                KernelIoControl(IOCTL_HAL_RELEASE_SYSINTR, 
                                  &(isrObj->dwSysIntr), 
                                  sizeof(DWORD), NULL, 
                                  0, NULL);
                SET_FAILURE_REASON;
            }
            if (DSP_SUCCEEDED(status))
            {
                /* InterruptInitialize */                
                osStatus = InterruptInitialize (isrObj->dwSysIntr, 
                                                isrObj->hIsrEvent,
                                                NULL, 0);        
                if (osStatus == FALSE)
                {
                    TRC_1PRINT (TRC_LEVEL7, "InterruptInitialize failed with error: %d\n",
                        osStatus) ;
                    status = DSP_EHANDLE;
                    /* release sysIntr */
                    KernelIoControl(IOCTL_HAL_RELEASE_SYSINTR, 
                                  &(isrObj->dwSysIntr), 
                                  sizeof(DWORD), NULL, 
                                  0, NULL);
                    /* close event handle */
                    CloseHandle(isrObj->hIsrEvent);
                    isrObj->hIsrEvent=NULL;
                    SET_FAILURE_REASON;
                }
                if (DSP_SUCCEEDED(status))
                {
                    /* Create a thread */                    
                    isrObj->hIsrThread = CreateThread ((LPSECURITY_ATTRIBUTES)NULL,
                                          0, (LPTHREAD_START_ROUTINE)ISR_Callback,
                                          isrObj,CREATE_SUSPENDED, NULL);
                    if (isrObj->hIsrThread == NULL)
                    {
                        TRC_0PRINT (TRC_LEVEL7, "CreateThread for IST failed \n") ;
                        status = DSP_EHANDLE;
                        /* remove the mapping */
                        InterruptDisable(isrObj->dwSysIntr);
                        /* release sysIntr */
                        KernelIoControl(IOCTL_HAL_RELEASE_SYSINTR, 
                                      &(isrObj->dwSysIntr), 
                                      sizeof(DWORD), NULL, 
                                      0, NULL);
                        /* close event handle */
                        CloseHandle(isrObj->hIsrEvent);
                        isrObj->hIsrEvent=NULL;

                        SET_FAILURE_REASON;
                    }
                    else

                    {       
                        bfrSz=sizeof(threadPRI);
                        tmpstatus=Reg_GetKeyValue(HKEY_LOCAL_MACHINE,"Drivers\\BuiltIn\\DSPLINK","ISR_THPRI",(Uint8 *)&threadPRI,&bfrSz);
                        if (!DSP_SUCCEEDED(tmpstatus))
                        {

                            threadPRI=ISR_THREAD_PRIORITY;
                        }
                        /* set the priority of the thread */

                        CeSetThreadPriority(isrObj->hIsrThread,threadPRI);                          
                        /* resume the thread */                        
                        ResumeThread (isrObj->hIsrThread) ;        
                    }
                }
            }
        }
       
       
        if (DSP_SUCCEEDED(status))  {
            /*
             *  Maintain the installed ISR object pointer locally for cleanup
             *  on ISR_Finalize.
             */
            ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] = isrObj ;

            /*
             *  The ISR is enabled by default upon install so indicate that in
             *  local state.
             */
            isrObj->enabled = TRUE ;
        }
        else
        {
            TRC_1PRINT (TRC_LEVEL7, "ISR_Install failed with error: %d\n",
                        status) ;
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }

    DBC_Ensure (   (   (DSP_SUCCEEDED (status))
                    && (   ISR_InstalledIsrs [isrObj->dspId][isrObj->irq]
                        == isrObj))
                || DSP_FAILED (status)) ;


    TRC_1LEAVE ("ISR_Install", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Uninstall
 *
 *  @desc   Uninstalls an ISR.
 *
 *  @modif  ISR_InstalledIsrs
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Uninstall (IN IsrObject * isrObj)
{
    DSP_STATUS      status   = DSP_SOK ;

    TRC_1ENTER ("ISR_Uninstall", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj!= NULL)
                 && (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] == isrObj)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] != isrObj) {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
    }
    else {
        if (isrObj->enabled == TRUE) {
            status = ISR_Disable (isrObj) ;
        }
        InterruptDisable (isrObj->dwSysIntr);
        
        if (DSP_SUCCEEDED (status)) {
            /* Release the Dynamic SYSINTR and the Thread */                
            KernelIoControl (IOCTL_HAL_RELEASE_SYSINTR, &(isrObj->dwSysIntr), 
                             sizeof(DWORD), NULL, 0, NULL);
            
            /* Terminate the IST Thread. We will not use the
             * TerminateThread API since it has certain caveats. Instead
             * we will set the Interrupt Handler Function to NULL and
             * set the Event object ourselves. Since the interrupt and
             * SYSINTR are disassociated with the KernelIoControl call
             * previously, we can be sure that the IST Thread woke up
             * because of the manual SetEvent Call. 
             */                                                
            isrObj->isrProcfn = NULL;
            SetEvent (isrObj->hIsrEvent);
            if (WaitForSingleObject(isrObj->hIsrThread, INFINITE) == WAIT_FAILED) {
                status = DSP_EFAIL;
                TRC_1PRINT(TRC_LEVEL7,"LINK: ISR_Uninstall: Wait failed for thread 0x%x\r\n", 
                                      isrObj->hIsrThread ) ;
            }
            
            if (DSP_SUCCEEDED(status)) {
                TRC_1PRINT(TRC_LEVEL2,"LINK: ISR_Uninstall: Thread 0x%x deleted\r\n", 
                                    isrObj->hIsrThread ) ;
            
                CloseHandle(isrObj->hIsrThread);
                isrObj->hIsrThread = NULL;        
                CloseHandle (isrObj->hIsrEvent);                                             
                isrObj->hIsrEvent=NULL;
            }            
            ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] = NULL ;
        }
        else {
            SET_FAILURE_REASON ;
        }
    }

    DBC_Ensure (   (   (DSP_SUCCEEDED (status))
                    && (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] == NULL))
                || DSP_FAILED (status)) ;

    TRC_1LEAVE ("ISR_Uninstall", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Disable
 *
 *  @desc   Disables the specified ISR.
 *          This function calls InterruptMask () to disable the ISR.
 *          InterruptMask() function doesn't return any value so this function
 *          assumes it was successful.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Disable (IN  IsrObject * isrObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("ISR_Disable", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj != NULL)
                 && (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] == isrObj)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] != isrObj) {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
    }
    else {    
        InterruptMask(isrObj->dwSysIntr,TRUE) ;
        isrObj->enabled = FALSE ;    
    }

    TRC_1LEAVE ("ISR_Disable", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_Enable
 *
 *  @desc   Enables the specified ISR.
 *          This function calls InterruptMask () to enable the ISR.
 *          InterruptMask() function doesn't return any value so this function
 *          assumes it was successful.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_Enable (IN  IsrObject * isrObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("ISR_Enable", isrObj) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;
    DBC_Require (   (isrObj != NULL)
                 && (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] == isrObj)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] != isrObj) {
            status = DSP_EACCESSDENIED ;
            SET_FAILURE_REASON ;
        }
    else {
        InterruptMask(isrObj->dwSysIntr,FALSE) ;
        isrObj->enabled = TRUE ;
    }

    TRC_1LEAVE ("ISR_Enable", status) ;

    return status ;
}


/** ============================================================================
 *  @func   ISR_GetState
 *
 *  @desc   Gets the state of an ISR.
 *          This function first checks if the ISR was installed using this
 *          subcomponent. If yes, then it checks the enabled flag of isrObject.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
ISR_GetState (IN  IsrObject *  isrObj,
              OUT ISR_State *  isrState)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_2ENTER ("ISR_GetState", isrObj, isrState) ;

    DBC_Require (ISR_IsInitialized == TRUE) ;
    DBC_Require (isrObj != NULL) ;
    DBC_Require (isrState != NULL) ;
    DBC_Require (IS_OBJECT_VALID (isrObj, SIGN_ISR)) ;

    if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == FALSE)  {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (isrState == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }

    if (DSP_SUCCEEDED (status)) {
        *isrState = ISR_Uninstalled ;

            if (ISR_InstalledIsrs [isrObj->dspId][isrObj->irq] == isrObj) {
                if (isrObj->enabled == TRUE) {
                    *isrState = ISR_Enabled ;
                }
                else {
                    *isrState = ISR_Disabled ;
                }
            }
        }

    TRC_1LEAVE ("ISR_GetState", status) ;

    return status ;
}


#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @func   ISR_Debug
 *
 *  @desc   Prints the current status of ISR objects in the system.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
ISR_Debug ()
{
    Int32  i = 0 ;
    Int32  j = 0 ;

    TRC_0ENTER ("ISR_Debug ()\n") ;

    DBC_Require (ISR_IsInitialized == TRUE) ;

    for (i = 0 ; i < MAX_DSPS ; i++) {
        for (j = 0 ; j < MAX_ISR ; j++) {
            if (ISR_InstalledIsrs [i][j] != NULL) {
                TRC_2PRINT (TRC_LEVEL1,
                            "ISR_InstalledIsrs [%d][0x%x]\n",
                            i, j) ;
                TRC_2PRINT (TRC_LEVEL1,
                            "   signature:   [%d][0x%x]\n",
                            i, ISR_InstalledIsrs [i][j]->signature) ;
                TRC_2PRINT (TRC_LEVEL1, "   interruptId: [%d][0x%x]\n",
                                        i, ISR_InstalledIsrs [i][j]->irq) ;
                TRC_2PRINT (TRC_LEVEL1, "   enabled:    [%d][0x%x]\n",
                                        i, ISR_InstalledIsrs [i][j]->enabled) ;
            }
        }
    }


    TRC_0PRINT (TRC_ENTER, "Leaving ISR_Debug ()\n") ;
}
#endif /* defined (DDSP_DEBUG) */


/** ----------------------------------------------------------------------------
 *  @func   ISR_Callback
 *
 *  @desc   Entry point for the IST. 
 *          This function waits for the event that is signalled when an interrupt occurs on
 *          the specified irq. This function then invokes the user specified interrupt 
 *          service routine.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
DWORD
ISR_Callback (LPVOID arg)
{
    DSP_STATUS   status     = DSP_SOK ;
    IsrObject *  isrObj     = (IsrObject *) (arg) ;
    Bool         isAsserted = FALSE ;
    int          handled    = 1 ;
    DWORD        dwRetval   = 0;

    
    /* For ever wait for the event to occur */
    do {
        dwRetval = WaitForSingleObject(isrObj->hIsrEvent, INFINITE);
        
        if (dwRetval != WAIT_OBJECT_0) {
            TRC_0PRINT(TRC_LEVEL7,"LINK: ISR_Callback: Wait Failed!\n");
            status = DSP_EFAIL;
            SET_FAILURE_REASON ;
            break;
        }
    
        if (IS_OBJECT_VALID (isrObj, SIGN_ISR) == TRUE) {
            if (isrObj->isrProcfn != NULL) {    
                isAsserted = TRUE;
                if (isrObj->checkFunc)
                    isAsserted = (*isrObj->checkFunc) (isrObj->param) ;
    
                if (isAsserted == TRUE) {
                    if (isrObj->enabled == TRUE) {
                        (*isrObj->isrProcfn) (isrObj->refdata) ;                            
                    }
                }   
                InterruptDone(isrObj->dwSysIntr) ;
            }
            else {
                /* Signalled to Stop the Thread */
                break;    
            }
        }
        else {
            status = DSP_EPOINTER;
            SET_FAILURE_REASON ;
        }
    } while (status == DSP_SOK);

    return ((DWORD)status);
    
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
