/** ============================================================================
 *  @file   mem.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/WinCE/
 *
 *  @desc   Implementation of sub-component MEM.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>
#include <kfuncs.h>
#include <ceddk.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <mem_os.h>
#include <mem.h>
#include <print.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_OSAL_MEM

/** ============================================================================
 *  @macro  USE_PROCESS_HEAP
 *
 *  @desc   Selects between the process heap or a fixed size heap
 *  ============================================================================
 */
#define USE_PROCESS_HEAP 1

/** ============================================================================
 *  @macro  DEFAULT_HEAP_MIN_SIZE
 *
 *  @desc   Minimum Default Heap Size Identifier.
 *  ============================================================================
 */
#define  DEFAULT_HEAP_MIN_SIZE   524288

/** ============================================================================
 *  @macro  DEFAULT_HEAP_MAX_SIZE
 *
 *  @desc   Maximum default Heap Size Identifier.
 *  ============================================================================
 */
#define  DEFAULT_HEAP_MAX_SIZE  1048576   

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_OSAL_MEM, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @name   MEM_IsInitialized
 *
 *  @desc   Flag to track initialization of the sub-component.
 *  ============================================================================
 */
STATIC Bool    MEM_IsInitialized  = FALSE   ;

/** ============================================================================
 *  @name   hMemDefaultHeap
 *
 *  @desc   HANDLE to the Memory Heap to support Default Memory Requests.
 *  ============================================================================
 */
STATIC HANDLE   hMemDefaultHeap = NULL ;

/** ============================================================================
 *  @name   MEM_SplAllocCount
 *
 *  @desc   Maintains the Byte Count of Specially Allocated Memory Regions.
 *  ============================================================================
 */
STATIC Uint32    MEM_SplAllocCount = 0 ;

#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @name   MEM_DefaultAlloc
 *
 *  @desc   Number of allocation with default type done so far.
 *  ============================================================================
 */
STATIC Uint32 MEM_DefaultAlloc = 0 ;

/** ============================================================================
 *  @name   MEM_DefaultFree
 *
 *  @desc   Number of deallocation with default type done so far.
 *  ============================================================================
 */
STATIC Uint32 MEM_DefaultFree = 0 ;

/** ============================================================================
 *  @name   MEM_SpecialAlloc
 *
 *  @desc   Number of allocation with non-default type done so far.
 *  ============================================================================
 */
STATIC Uint32 MEM_SpecialAlloc = 0 ;

/** ============================================================================
 *  @name   MEM_SpecialFree
 *
 *  @desc   Number of deallocation with non-default type done so far.
 *  ============================================================================
 */
STATIC Uint32 MEM_SpecialFree = 0 ;
#endif


/** ============================================================================
 *  @func   MEM_Initialize
 *
 *  @desc   Initializes the MEM sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("MEM_Initialize") ;

    DBC_Require (MEM_IsInitialized == FALSE) ;

#if USE_PROCESS_HEAP
    /* The process heap does not suffer the restrictions of a fixed
     * sized heap.  Loading DSP images with large symbol tables requires
     * large allocs.
     */
    hMemDefaultHeap = GetProcessHeap();
#else
    /* Allocate the Default Heap 
     * 
     * The Default Heap will be used for all the local
     * object allocations. We will initially start
     * with a 128 KB Heap and the max size is
     * set to 256 KB currently
     */
    hMemDefaultHeap = HeapCreate (HEAP_NO_SERIALIZE, 
                                  DEFAULT_HEAP_MIN_SIZE,
                                  DEFAULT_HEAP_MAX_SIZE); 
#endif
    if (hMemDefaultHeap == NULL) {
        status = DSP_EMEMORY ;
        SET_FAILURE_REASON;      
        TRC_1PRINT(TRC_LEVEL7,"LINK: MEM_Initialize Failed 0x%X.\r\n", 
                status ) ;
    }                                       

    MEM_IsInitialized = TRUE ;

    TRC_1LEAVE ("MEM_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Finalize
 *
 *  @desc   Releases resources used by this sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("MEM_Finalize") ;

    DBC_Require (MEM_IsInitialized == TRUE) ;

#if USE_PROCESS_HEAP
    hMemDefaultHeap = NULL;
#else
    /* Destroy the Default Heap */
    if (hMemDefaultHeap != NULL) {
        HeapDestroy (hMemDefaultHeap);
    }                                
#endif
    MEM_IsInitialized = FALSE ;

    TRC_1LEAVE ("MEM_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Alloc
 *
 *  @desc   Allocates the specified number of bytes.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Alloc (OUT Void ** ptr, IN Uint32 cBytes, IN OUT Pvoid arg)
{
    DSP_STATUS       status   = DSP_SOK ;
    MemAllocAttrs *  allocArg = NULL    ;
    BOOL             retValue           ;
    DMA_ADAPTER_OBJECT Adapter          ;
    Pvoid             bufPtr = NULL     ;
    PHYSICAL_ADDRESS  phyAddr           ;

    TRC_3ENTER ("MEM_Alloc", ptr, cBytes, arg) ;

    DBC_Require (ptr != NULL) ;
    DBC_Require (MEM_IsInitialized == TRUE) ;
    DBC_Require (cBytes != 0) ;

    /*  ------------------------------------------------------------------------
     *  Validate arguments
     *  ------------------------------------------------------------------------
     */
    if (ptr == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else if (cBytes == 0) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;

        *ptr = NULL ;
    }
    else {
        if (arg == MEM_DEFAULT) {
            /*  ----------------------------------------------------------------
             *  Allocations from 'default' memory area
             *  ----------------------------------------------------------------
             */
            *ptr = HeapAlloc (hMemDefaultHeap, HEAP_ZERO_MEMORY, (DWORD)cBytes) ;
            if (*ptr == NULL) {
                TRC_1PRINT(TRC_LEVEL7,"LINK: MEM_Alloc for 0x%X Bytes failed.\r\n", 
                        cBytes ) ;
                status = DSP_EMEMORY ;
                SET_FAILURE_REASON ;
            }
#if defined (DDSP_DEBUG)
            else {
                MEM_DefaultAlloc++ ;
            }
#endif    /* if defined (DDSP_DEBUG) */
        }
        else {
            /*  ----------------------------------------------------------------
             *  OS dependent allocation from 'special' memory area(s).
             *  ----------------------------------------------------------------
             */
            allocArg = (MemAllocAttrs *) arg ;

            /* For specifying the Physical Address to Virtual
             * Address Translation, specify the PAGE_PHYSICAL Bit
             * Flag. However, VirtualCopy has a restriction that
             * when PAGE_PHYSICAL is called, the Physical Address
             * should be rounded off to multiple of 256 Bytes.
             */
             Adapter.ObjectSize = sizeof(DMA_ADAPTER_OBJECT);
             Adapter.InterfaceType = (INTERFACE_TYPE)0x0;
             Adapter.BusNumber     = 0x0 ;
             bufPtr = HalAllocateCommonBuffer(&Adapter,
                             cBytes, (PPHYSICAL_ADDRESS)&phyAddr, FALSE);

            if (bufPtr != NULL) {
                *ptr = (void *)(bufPtr);
                allocArg->physicalAddress = (Uint32 *)phyAddr.LowPart ;
                
                TRC_3PRINT (TRC_LEVEL2,
                    "MEM_Alloc: Virt 0x%X. Phys 0x%X Size 0x%X Bytes.\r\n",
                    bufPtr, allocArg->physicalAddress, cBytes) ;
            }                                 
            else {
                status = DSP_EMEMORY ;
                SET_FAILURE_REASON ;
                TRC_2PRINT(TRC_LEVEL7,"LINK: MEM_Alloc for 0x%X Bytes failed. Error = 0x%X.\r\n", 
                                       cBytes, GetLastError() ) ;
                
            }
#if defined (DDSP_DEBUG)
            if (bufPtr != NULL) {
                MEM_SpecialAlloc++ ;
                /* Increment the Special Allocate Byte Count */
                MEM_SplAllocCount += cBytes;
            }
#endif    /* if defined (DDSP_DEBUG) */
        }
    }

    DBC_Ensure (   ((ptr == NULL) && DSP_FAILED (status))
                || ((ptr != NULL) && (*ptr != NULL) && DSP_SUCCEEDED (status))
                || ((ptr != NULL) && (*ptr == NULL) && DSP_FAILED (status))) ;

    TRC_1LEAVE ("MEM_Alloc", status) ;

    return status ;
}

/** ============================================================================
 *  @func   MEM_Calloc
 *
 *  @desc   Allocates the specified number of bytes and memory is set to zero.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Calloc (OUT Void ** ptr, IN Uint32 cBytes, IN OUT Pvoid arg)
{
    DSP_STATUS status = DSP_SOK ;
    Uint32       i                ;

    TRC_3ENTER ("MEM_Calloc", ptr, cBytes, arg) ;

    DBC_Require (ptr != NULL) ;
    DBC_Require (MEM_IsInitialized == TRUE) ;
    DBC_Require (cBytes != 0) ;

    status = MEM_Alloc (ptr, cBytes, arg) ;

    if (DSP_SUCCEEDED (status)) {
        for (i = 0 ; i < cBytes ; i++) {
            (*(Uint8 **) ptr)[i] = 0 ;
        }
    }

    DBC_Ensure (   ((ptr == NULL) && DSP_FAILED (status))
                || ((ptr != NULL) && (*ptr != NULL) && DSP_SUCCEEDED (status))
                || ((ptr != NULL) && (*ptr == NULL) && DSP_FAILED (status))) ;

    TRC_1LEAVE ("MEM_Calloc", status) ;

    return  status ;
}


/** ============================================================================
 *  @func   MEM_Free
 *
 *  @desc   Frees up the specified chunk of memory.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Free (IN Pvoid * ptr, IN Pvoid arg)
{
    DSP_STATUS       status   = DSP_SOK ;
    MemFreeAttrs *   freeArg  = NULL    ;
    DMA_ADAPTER_OBJECT Adapter          ;
    Pvoid             bufPtr = NULL     ;
    PHYSICAL_ADDRESS  phyAddr           ;

    TRC_2ENTER ("MEM_Free", ptr, arg) ;

    DBC_Require (ptr != NULL) ;
    DBC_Require (MEM_IsInitialized == TRUE) ;

    /*  ------------------------------------------------------------------------
     *  Validate arguments
     *  ------------------------------------------------------------------------
     */
    if (ptr == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (*ptr == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        if (arg == MEM_DEFAULT) {
            /*  ----------------------------------------------------------------
             *  Free allocations from 'default' memory area
             *  ----------------------------------------------------------------
             */
            HeapFree (hMemDefaultHeap, 0, (LPVOID)(*ptr)) ;

#if defined (DDSP_DEBUG)
            MEM_DefaultFree++ ;
#endif    /* if defined (DDSP_DEBUG) */
        }
        else {
            /*  ----------------------------------------------------------------
             *  Free OS dependent allocation from 'special' memory area(s).
             *  ----------------------------------------------------------------
             */
            freeArg = (MemFreeAttrs *) arg ;

            TRC_3PRINT (TRC_LEVEL2,
                    "MEM_Free: Virt 0x%X. Phys 0x%X Size 0x%X Bytes.\r\n",
                    *ptr, freeArg->physicalAddress, freeArg->size) ;

            if (MEM_SplAllocCount > 0) {
                phyAddr.LowPart  = (DWORD) freeArg->physicalAddress ;
                phyAddr.HighPart = 0;
                HalFreeCommonBuffer (&Adapter, freeArg->size,
                                     phyAddr,  (PVOID) *ptr,
                                     FALSE) ;  
                MEM_SplAllocCount -= freeArg->size;
#if defined (DDSP_DEBUG)
            MEM_SpecialFree++ ;
#endif    /* if defined (DDSP_DEBUG) */
        }
        }                      
        *ptr = NULL ;
    }

    TRC_1LEAVE ("MEM_Free", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Map
 *
 *  @desc   Maps a memory area into virtual space.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Map (IN OUT MemMapInfo * mapInfo)
{
    PHYSICAL_ADDRESS PhysicalIoBase;
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("MEM_Map", mapInfo) ;

    DBC_Require (mapInfo != NULL) ;
    mapInfo->dst = 0 ;
    
    if (mapInfo != NULL) {
        /*
                       * We will be using the MmMapIoSpace() call since
                       * whichever client who wants to access DSP
                       * Memory will go through the DSP LINK Stream
                       * Driver. Hence the Virtual Address returned
                       * by this call is specific to DEVICE.EXE
                      */
        PhysicalIoBase.LowPart = (mapInfo->src);
        PhysicalIoBase.HighPart = 0;
        
        if (mapInfo->memAttrs == MEM_UNCACHED) {
          mapInfo->dst = (Uint32) MmMapIoSpace( PhysicalIoBase,
                                                mapInfo->size, FALSE );
        }
        else if (mapInfo->memAttrs == MEM_CACHED) {
          mapInfo->dst = (Uint32) MmMapIoSpace( PhysicalIoBase,
                                                mapInfo->size, TRUE );
        }
        else {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }

        if (mapInfo->dst == 0) {
            status = DSP_EMEMORY ;
            SET_FAILURE_REASON ;
        }
        else {
            TRC_2PRINT (TRC_LEVEL2,
                    "MEM_Map: Map 0x%X to 0x%X.\r\n",
                    mapInfo->src, mapInfo->dst ) ;
        }
    }
    else {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }

    DBC_Ensure (  (DSP_SUCCEEDED (status) && (mapInfo->dst != 0))
               || (DSP_FAILED (status)    && (mapInfo->dst == 0))) ;

    TRC_1LEAVE ("MEM_Map", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Unmap
 *
 *  @desc   Unmaps a virtual memory area.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Unmap (IN MemUnmapInfo * unmapInfo)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("MEM_Unmap", unmapInfo) ;

    DBC_Require (unmapInfo != NULL) ;

    if (unmapInfo != NULL) {
        DBC_Assert (unmapInfo->addr != 0) ;
        //memset ((void *)unmapInfo->addr, 0, unmapInfo->size) ;
        TRC_2PRINT (TRC_LEVEL2,
                    "MEM_Unmap: 0x%X Size 0x%X.\r\n",
                    unmapInfo->addr, unmapInfo->size ) ;            
        MmUnmapIoSpace ((PVOID) unmapInfo->addr, unmapInfo->size) ;
    }
    else {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("MEM_Unmap", status) ;

    return status ;
}


/** ============================================================================
 *  @func   MEM_Copy
 *
 *  @desc   Copies the data between memory areas.
 *          (The endianism is currently not used.)
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
MEM_Copy (IN Uint8 * dst, OUT Uint8 * src, IN Uint32 len, IN Endianism end)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_4ENTER ("MEM_Copy", dst, src, len, end) ;

    memcpy (dst, src, len) ;

    TRC_1LEAVE ("MEM_Copy", status) ;

    return status ;
}


#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @func   MEM_Debug
 *
 *  @desc   Prints debug information for MEM.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
MEM_Debug ()
{
    TRC_0ENTER ("MEM_Debug") ;

    TRC_4PRINT (TRC_LEVEL4,
                "    MEM_DefaultAlloc [%d]\n"
                "    MEM_DefaultFree  [%d]\n"
                "    MEM_SpecialAlloc [%d]\n"
                "    MEM_SpecialFree  [%d]\n",
                 MEM_DefaultAlloc, MEM_DefaultFree,
                 MEM_SpecialAlloc, MEM_SpecialFree) ;

    TRC_0LEAVE ("MEM_Debug") ;
}
#endif /* defined (DDSP_DEBUG) */


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
