/** ============================================================================
 *  @file   sync.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/WinCE/
 *
 *  @desc   Implementation of sub-component SYNC.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>
#include <pkfuncs.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_SYNC

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_OSAL_SYNC, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @name   SYNC_SpinLockObject
 *
 *  @desc   This object is used by spinlock APIs.
 *
 *  @field  lock
 *              Critical Section handle to lock object used by OS. 
 *  @see    None
 *  ============================================================================
 */
typedef struct SYNC_SpinLockObject_tag {
    CRITICAL_SECTION lock;
} SYNC_SpinLockObject ;


/** ============================================================================
 *  @name   SyncEvObject
 *
 *  @desc   This object is used for various event related API.
 *
 *  @field  signature
 *              For identification of this object.
 *  @field  hEvent
 *              Handle for the event.
 *  @field  timeoutOccured
 *              To protect write/read access to event.
 *
 *  @see    None
 *  ============================================================================
 */
struct SyncEvObject_tag {
    Uint32            signature       ;
    HANDLE            hEvent          ; 
    Bool             timeoutOccurred   ;
} ;


/** ============================================================================
 *  @name   SyncCsObject
 *
 *  @desc   This object is used by various CS API's.
 *
 *  @field  signature
 *              For identification of this object.
 *  @field  cs
 *              OS specific lock that is used to implement CS API.
 *
 *  @see    None
 *  ============================================================================
 */
struct SyncCsObject_tag {
    Uint32              signature   ;
    CRITICAL_SECTION    cs          ;
} ;


/** ============================================================================
 *  @name   SyncSemObject
 *
 *  @desc   This object is used by various SEM API's.
 *
 *  @field  signature
 *              For identification of this object.
 *  @field  semType
 *              Indicates the type of the semaphore (binary or counting).
 *  @field  list
 *              List of waiting processes.
 *  @field  value
 *              Current status of semaphore (0,1) - binary and (0-n) counting.
 *  @field  lock
 *              Indicates that timeout had occurred.
 *
 *  @see    None
 *  ============================================================================
 */
#define SEM_MAX_COUNT 200

struct SyncSemObject_tag {
    Uint32            signature         ;
    SyncSemType       semType           ;
    HANDLE            sem               ;
    Uint32            isSemAvailable    ;
    Bool              timeoutOccurred    ;
} ;


/** ----------------------------------------------------------------------------
 *  @name   SYNC_IsInitialized
 *
 *  @desc   Indicates if the module has been initialized or not.
 *  ----------------------------------------------------------------------------
 */
STATIC Bool SYNC_IsInitialized = FALSE ;

/** ----------------------------------------------------------------------------
 *  @name   SYNC_PreemptLock
 *
 *  @desc   Spin lock structure to provide protection from IRQ, DPC, TASKs,
 *          tasks and tasklets.
 *  ----------------------------------------------------------------------------
 */
STATIC CRITICAL_SECTION SYNC_PreemptLock ;

/** ----------------------------------------------------------------------------
 *  @name   SYNC_ProtectCsObj
 *
 *  @desc   Handle to the critical section object used for protecting against
 *          other tasks during SYNC_ProtectionStart ()/SYNC_ProtectionEnd ()
 *          calls.
 *  ----------------------------------------------------------------------------
 */
STATIC SyncCsObject * SYNC_ProtectCsObj = NULL ;


/** ============================================================================
 *  @func   SYNC_Initialize
 *
 *  @desc   Initialize SYNC sub-component by allocating all resources.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("SYNC_Initialize") ;

    DBC_Require (SYNC_IsInitialized == FALSE) ;

    /*  Create the SYNC protection CS object  */
    status = SYNC_CreateCS (&SYNC_ProtectCsObj) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    /* Initialize the global general lock */
    InitializeCriticalSection(&SYNC_PreemptLock);

    SYNC_IsInitialized = TRUE ;

    TRC_1LEAVE ("SYNC_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_Finalize
 *
 *  @desc   Releases resources used by the sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("SYNC_Finalize") ;

    DBC_Require (SYNC_IsInitialized == TRUE) ;

    /*  Delete the SYNC protection CS object  */
    if (SYNC_ProtectCsObj != NULL) {
        status = SYNC_DeleteCS (SYNC_ProtectCsObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
        SYNC_ProtectCsObj = NULL ;
    }

    DeleteCriticalSection(&SYNC_PreemptLock);

    SYNC_IsInitialized = FALSE ;

    TRC_1LEAVE ("SYNC_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_OpenEvent
 *
 *  @desc   Creates and initializes an event object for thread
 *          synchronization. The event is initialized to a
 *          non-signaled state.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_OpenEvent (OUT SyncEvObject ** event,
                IN  SyncAttrs *     attr)
{
    DSP_STATUS     status = DSP_SOK ;
    SyncEvObject * _event           ;

    TRC_2ENTER ("SYNC_OpenEvent", event, attr) ;

    DBC_Require (event != NULL) ;
    DBC_Require (attr != NULL) ;

    if (attr == NULL) {
        if (event != NULL) {
            *event = NULL ;
        }
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else if (event == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status = MEM_Calloc ((Void **) event, sizeof (SyncEvObject),
                            MEM_DEFAULT) ;
        if (DSP_SUCCEEDED (status)) {
            _event = *event ;
            /* Create the event */
            _event->hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
            if (_event->hEvent != NULL)
            {
                _event->signature = SIGN_SYNC ;
                _event->timeoutOccurred = FALSE;
            }
            else
            {
               FREE_PTR(*event);    
               status = DSP_EFAIL;
               SET_FAILURE_REASON;
            }
        }
        else {
            SET_FAILURE_REASON ;
            *event = NULL ;
        }
    }

    DBC_Ensure (   ((event == NULL) && DSP_FAILED (status))
                 || ((*event == NULL) && DSP_FAILED (status))
                 || (   ((*event)->signature == SIGN_SYNC)
                     && DSP_SUCCEEDED (status))) ;

    TRC_1LEAVE ("SYNC_OpenEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_CloseEvent
 *
 *  @desc   Closes the handle corresponding to an event. It also frees the
 *          resources allocated, if any, during call to SYNC_OpenEvent ().
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_CloseEvent (IN SyncEvObject * event)
{
    DSP_STATUS   status   = DSP_SOK    ;
    BOOL         osStatus;

    TRC_1ENTER ("SYNC_CloseEvent", event) ;

    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;
    DBC_Require (event != NULL) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {        
        event->signature = SIGN_NULL ;
    }
    else {
        SET_FAILURE_REASON ;
    }

    if (DSP_SUCCEEDED (status)) {
        osStatus=CloseHandle(event->hEvent);
        if (osStatus == FALSE)
        {
            TRC_0PRINT(TRC_LEVEL7,"SYNC_CloseEvent: close of event handle failed\r\n");
            status = DSP_EHANDLE;
            SET_FAILURE_REASON ;
        }
    }
    if (DSP_SUCCEEDED (status)) {
        status = FREE_PTR (event) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("SYNC_CloseEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_ResetEvent
 *
 *  @desc   Reset the synchronization object to non-signaled state.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_ResetEvent (IN SyncEvObject * event)
{
    DSP_STATUS   status   = DSP_SOK    ;
    BOOL         osStatus;

    TRC_1ENTER ("SYNC_ResetEvent", event) ;

    DBC_Require (event != NULL) ;
    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        osStatus=ResetEvent(event->hEvent);
        if (osStatus == FALSE)
        {
            status=DSP_EHANDLE;
            SET_FAILURE_REASON;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_ResetEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_SetEvent
 *
 *  @desc   Set the state of synchronization object to signaled and unblock all
 *          threads waiting for it.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_SetEvent (IN SyncEvObject * event)
{
    DSP_STATUS status   = DSP_SOK    ;

    TRC_1ENTER ("SYNC_SetEvent", event) ;

    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;
    DBC_Require (event != NULL) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        if (!SetEvent(event->hEvent))
        {
            status=DSP_EFAIL;
            SET_FAILURE_REASON;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_SetEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_WaitOnEvent
 *
 *  @desc   Wait for an event to be signaled for a specified amount of time.
 *          It is possible to wait infinitely. This function must 'block'
 *          and not 'spin'.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_WaitOnEvent (IN SyncEvObject * event, IN Uint32 timeout)
{
    DSP_STATUS  status = DSP_ENOTIMPL;
    DWORD       wait_timeout=timeout;

    TRC_2ENTER ("SYNC_WaitOnEvent", event, timeout) ;

    DBC_Require (event != NULL) ;
    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
		if (timeout == SYNC_WAITFOREVER) wait_timeout = INFINITE;
		if (timeout == SYNC_NOWAIT) wait_timeout = 0;
		{
			DWORD dwWaitResult = WaitForSingleObject(event->hEvent, wait_timeout);
			switch (dwWaitResult) {
			case WAIT_FAILED:
				status = SYNC_E_FAIL;
				SET_FAILURE_REASON;
				break;
			case WAIT_OBJECT_0 :
				break;
			case WAIT_TIMEOUT:
                if (timeout == SYNC_NOWAIT) 
                    status = SYNC_E_FAIL;
                else
                    status = DSP_ETIMEOUT; 
				event->timeoutOccurred = TRUE;
				break;
			}
		}
	}
	else {
		SET_FAILURE_REASON;
	}
				
    TRC_1LEAVE ("SYNC_WaitOnEvent", status) ;

	return status;
}


/** ============================================================================
 *  @func   SYNC_WaitOnMultipleEvents
 *
 *  @desc   Waits on muliple events. Returns when any of the event is set.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_WaitOnMultipleEvents (IN  SyncEvObject **   syncEvents,
                           IN  Uint32            count,
                           IN  Uint32            timeout,
                           OUT Uint32 *          index)
{
    DSP_STATUS      status = DSP_SOK ;

    TRC_4ENTER ("SYNC_WaitOnMultipleEvents", syncEvents, count,
                                             timeout, index) ;
    DBC_Require (syncEvents != NULL) ;
    DBC_Require (index != NULL) ;

    /* Not Implemented */
    status = DSP_ENOTIMPL ;
    SET_FAILURE_REASON ;

    TRC_1LEAVE ("SYNC_WaitOnMultipleEvents", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_CreateCS
 *
 *  @desc   Creates the Critical section structure.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_CreateCS (OUT SyncCsObject ** cSObj)
{
    DSP_STATUS   status   = DSP_SOK    ;

    TRC_1ENTER ("SYNC_CreateCS", cSObj) ;

    DBC_Require (cSObj != NULL) ;

    if (cSObj == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }

    if (DSP_SUCCEEDED (status)) {
        status =  MEM_Calloc ((Void **) cSObj, sizeof (SyncCsObject),
                              MEM_DEFAULT) ;
    }

    if (DSP_SUCCEEDED (status)) {
        InitializeCriticalSection (&((*cSObj)->cs)) ;
        (*cSObj)->signature = SIGN_SYNC ;
    }
    else {
        SET_FAILURE_REASON ;
    }

    DBC_Ensure (   ((cSObj == NULL) && DSP_FAILED (status))
                 || ((*cSObj == NULL) && DSP_FAILED (status))
                 || (   ((*cSObj)->signature == SIGN_SYNC)
                     && DSP_SUCCEEDED (status))) ;

    TRC_1LEAVE ("SYNC_CreateCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_DeleteCS
 *
 *  @desc   Deletes the critical section object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_DeleteCS (IN SyncCsObject * cSObj)
{
    DSP_STATUS   status   = DSP_SOK    ;

    TRC_1ENTER ("SYNC_DeleteCS", cSObj) ;

    DBC_Require (cSObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (cSObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (cSObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        cSObj->signature = SIGN_NULL ;
        DeleteCriticalSection(&cSObj->cs);            
        status = FREE_PTR (cSObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_DeleteCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_EnterCS
 *
 *  @desc   This function enters the critical section that is passed as
 *          argument to it. After successful return of this function no other
 *          thread can enter until this thread exit CS by calling SYNC_LeaveCS
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_EnterCS (IN SyncCsObject * cSObj)
{
    DSP_STATUS   status   = DSP_SOK    ;
    
    TRC_1ENTER ("SYNC_EnterCS", cSObj) ;

    DBC_Require (cSObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (cSObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (cSObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
       EnterCriticalSection(&cSObj->cs) ;
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_EnterCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_LeaveCS
 *
 *  @desc   This function makes critical section available for other threads
 *          to enter.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_LeaveCS (IN SyncCsObject * cSObj)
{
    DSP_STATUS   status   = DSP_SOK    ;

    TRC_1ENTER ("SYNC_LeaveCS", cSObj) ;

    DBC_Require (cSObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (cSObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (cSObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        LeaveCriticalSection(&cSObj->cs) ;
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_LeaveCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_CreateSEM
 *
 *  @desc   Creates the semaphore object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_CreateSEM (OUT SyncSemObject ** semObj, IN SyncAttrs * attr)
{
    DSP_STATUS   status   = DSP_SOK ;

    TRC_2ENTER ("SYNC_CreateSEM", semObj, attr) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (attr != NULL) ;

    if ((semObj == NULL) || (attr == NULL)) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (   ((SyncSemType)(attr->flag) != SyncSemType_Binary)
             && ((SyncSemType)(attr->flag) != SyncSemType_Counting))  {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status =  MEM_Calloc ((Void **) semObj, sizeof (SyncSemObject),
                              MEM_DEFAULT) ;

        if (DSP_SUCCEEDED (status)) {
            DWORD max_count=1;
            (*semObj)->semType = (SyncSemType)(attr->flag) ;
            (*semObj)->isSemAvailable = FALSE ;
            //(*semObj)->lock = SYNC_SpinLockCreate () ;
            (*semObj)->timeoutOccurred=FALSE;
            max_count=(((*semObj)->semType)==SyncSemType_Binary)? 1 : SEM_MAX_COUNT;
            (*semObj)->sem = CreateSemaphore(NULL,0,max_count,NULL);
            if (NULL == (*semObj)->sem)
            {
                status=DSP_EFAIL;
                FREE_PTR(*semObj);
                SET_FAILURE_REASON;
            }               
            else
            {
                (*semObj)->signature = SIGN_SYNC ;
            }
        }
        else {
            SET_FAILURE_REASON ;
        }
    }

    if ((DSP_FAILED (status)) && (semObj != NULL)) {
        *semObj = NULL ;
    }

    DBC_Ensure (   ((semObj == NULL) && DSP_FAILED (status))
                 || ((*semObj == NULL) && DSP_FAILED (status))
                 || (   ((*semObj)->signature == SIGN_SYNC)
                     && DSP_SUCCEEDED (status))) ;

    TRC_1LEAVE ("SYNC_CreateSEM", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_DeleteSEM
 *
 *  @desc   Deletes the semaphore object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_DeleteSEM (IN SyncSemObject * semObj)
{
    DSP_STATUS   status   = DSP_SOK ;

    TRC_1ENTER ("SYNC_DeleteSEM", semObj) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (semObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (semObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        //SYNC_SpinLockDelete (semObj->lock) ;
        if (!CloseHandle(semObj->sem)) {
			status = DSP_EFAIL;
			SET_FAILURE_REASON;
		}			
        semObj->signature = SIGN_NULL ;
        status = FREE_PTR (semObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_DeleteSEM", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_WaitSEM
 *
 *  @desc   This function waits on the semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_WaitSEM (IN SyncSemObject * semObj, IN Uint32  timeout)
{
    DSP_STATUS  status = DSP_SOK;
    DWORD       wait_timeout = timeout;

    TRC_2ENTER ("SYNC_WaitSEM", semObj, timeout) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (semObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (semObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        semObj->timeoutOccurred = FALSE ;
		
        if (semObj->semType == SyncSemType_Binary) {
			BOOL semAvail = FALSE;
            Uint32 flags = SYNC_SpinLockStart () ;
            if (semObj->isSemAvailable == TRUE) {
				semAvail = TRUE;
                semObj->isSemAvailable = FALSE ;
                /* Make sure semaphore is still in reset state */
                WaitForSingleObject(semObj->sem, 0);
            }
            SYNC_SpinLockEnd (flags) ;
			if (semAvail) return status;
		}

		if (timeout == SYNC_WAITFOREVER) wait_timeout = INFINITE;
		if (timeout == SYNC_NOWAIT) wait_timeout = 0;
		{
			DWORD dwWaitResult = WaitForSingleObject(semObj->sem, timeout);
			switch (dwWaitResult) {
			case WAIT_FAILED:
				status = SYNC_E_FAIL;
				SET_FAILURE_REASON;
				break;
			case WAIT_OBJECT_0 :
				if (semObj->semType == SyncSemType_Binary) {
					Uint32 flags = SYNC_SpinLockStart () ;
					semObj->isSemAvailable = FALSE ;
					SYNC_SpinLockEnd (flags) ;
				}				
				break;
			case WAIT_TIMEOUT:
                if (timeout == SYNC_NOWAIT) 
                    status = SYNC_E_FAIL;
                else
                    status = DSP_ETIMEOUT;                
				semObj->timeoutOccurred = TRUE;
                SET_FAILURE_REASON;
				break;
			}
		}
	}
		
   return status;
}


/** ============================================================================
 *  @func   SYNC_SignalSEM
 *
 *  @desc   This function signals the semaphore and makes it available for other
 *          threads.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_SignalSEM (IN SyncSemObject * semObj)
{
    DSP_STATUS   status    = DSP_SOK   ;

    TRC_1ENTER ("SYNC_SignalSEM", semObj) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (semObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (semObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        if (semObj->semType == SyncSemType_Counting) {
			if (!ReleaseSemaphore(semObj->sem, 1, NULL)) {
				RETAILMSG(1, (L"SYNC_SignalSEM: ReleaseSemaphore failure(%d)\r\n", 
					GetLastError()));
				status = DSP_EFAIL;
				SET_FAILURE_REASON;
			}
		}
		else {
			BOOL signalSem = FALSE;
            DWORD flags = SYNC_SpinLockStart () ;
            if (semObj->isSemAvailable != TRUE) {
                semObj->isSemAvailable = TRUE ;
                signalSem = TRUE ;
            }
            SYNC_SpinLockEnd (flags) ;

            if (signalSem == TRUE) {
                /* Now make binary semaphore 1 */
				if (!ReleaseSemaphore(semObj->sem, 1, NULL)) {
					RETAILMSG(1, (L"SYNC_SignalSEM: ReleaseSemaphore failure(%d)\r\n", 
						GetLastError()));
					status = DSP_EFAIL;
					SET_FAILURE_REASON;
				}
            }
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_SignalSEM", status) ;

   return status;
}


/** ============================================================================
 *  @func   SYNC_ProtectionStart
 *
 *  @desc   Marks the start of protected code execution.
 *          DSP/BIOS Link implements DPC using tasklets.
 *          This function acheives protection by disabling DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_ProtectionStart ()
{
    DSP_STATUS status = DSP_SOK ;
    TRC_0ENTER ("SYNC_ProtectionStart") ;

    status = SYNC_EnterCS (SYNC_ProtectCsObj) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    /* Disable all DPC */
    DPC_Disable () ;

    TRC_0LEAVE ("SYNC_ProtectionStart") ;
}


/** ============================================================================
 *  @func   SYNC_ProtectionEnd
 *
 *  @desc   Marks the end of protected code execution.
 *          DSP/BIOS Link implements DPC using tasklets.
 *          This function enables DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_ProtectionEnd ()
{
    DSP_STATUS status = DSP_SOK ;

    TRC_0ENTER ("SYNC_ProtectionEnd") ;

    /* Enable all DPC */
    DPC_Enable () ;

    status = SYNC_LeaveCS (SYNC_ProtectCsObj) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    TRC_0LEAVE ("SYNC_ProtectionEnd") ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockCreate
 *
 *  @desc   Creates the Spinlock Object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Pvoid
SYNC_SpinLockCreate (Void)
{
    DSP_STATUS            status = DSP_SOK ;
    Pvoid                 spinLockHandle = NULL ;
    SYNC_SpinLockObject * obj = NULL ;

    TRC_0ENTER ("SYNC_SpinLockCreate") ;

    status = MEM_Alloc ((Pvoid *) &spinLockHandle,
                        sizeof (SYNC_SpinLockObject),
                        MEM_DEFAULT) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }
    else {
        obj = (SYNC_SpinLockObject *) spinLockHandle ;
        InitializeCriticalSection(&obj->lock);        
    }

    TRC_1LEAVE ("SYNC_SpinLockCreate", spinLockHandle) ;

    return spinLockHandle ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockDelete
 *
 *  @desc   Deletes the Spinlock Object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_SpinLockDelete (IN Pvoid spinLockHandle)
{
    TRC_1ENTER ("SYNC_SpinLockDelete", spinLockHandle) ;

    if (spinLockHandle != NULL) {
        SYNC_SpinLockObject * obj = NULL ;    
        obj = (SYNC_SpinLockObject *) spinLockHandle ;
		DeleteCriticalSection(&obj->lock);
        FREE_PTR (spinLockHandle) ;
    }

    TRC_0LEAVE ("SYNC_SpinLockDelete") ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockStartEx
 *
 *  @desc   Begin protection of code through spin lock with all ISRs disabled.
 *          Calling this API protects critical regions of code from preemption
 *          by tasks, DPCs and all interrupts.
 *          This API can be called from DPC context.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Uint32
SYNC_SpinLockStartEx (IN     Pvoid spinLockHandle)
{
    TRC_1ENTER ("SYNC_SpinLockStartEx", spinLockHandle) ;

    if (spinLockHandle != NULL) {
        SYNC_SpinLockObject * obj
			= (SYNC_SpinLockObject *) spinLockHandle ;
		EnterCriticalSection (&obj->lock) ;
    }
    else {
		EnterCriticalSection (&SYNC_PreemptLock) ;
    }

    TRC_0LEAVE ("SYNC_SpinLockStartEx") ;

    return 0 ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockEndEx
 *
 *  @desc   End protection of code through spin lock with all ISRs disabled.
 *          This API can be called from DPC context.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_SpinLockEndEx (IN     Pvoid spinLockHandle, IN     Uint32 irqFlags)
{
    if(!SYNC_IsInitialized ) return;

    TRC_1ENTER ("SYNC_SpinLockEndEx", spinLockHandle) ;

    if (spinLockHandle != NULL) {
        SYNC_SpinLockObject * obj
			= (SYNC_SpinLockObject *) spinLockHandle ;

		LeaveCriticalSection(&obj->lock);
    }
    else {
		LeaveCriticalSection(&SYNC_PreemptLock) ;
    }

    TRC_0LEAVE ("SYNC_SpinLockEndEx") ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockEnd
 *
 *  @desc   Pause for number microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_UDelay (IN Uint32 numUSec)
{
    TRC_1ENTER ("SYNC_USec", numUSec) ;

    Sleep ((numUSec+999)/1000) ; // sleep needs milliseconds. some resolution is lost here

    TRC_0LEAVE ("SYNC_UDelay") ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
