/** ============================================================================
 *  @file   message_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/message_multidsp/PrOS/
 *
 *  @desc   OS specific implementation of functions used by the message
 *          sample application.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <itron.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>

/*  ----------------------------------- Application Header            */
#include <message.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

#if defined (MESSAGE_PROFILE)

/** ============================================================================
 *  @name   initialMilisecTime
 *
 *  @desc   This varibale stores the value of  time in milliseconds.
 *          It is 64 bit size.
 *  ============================================================================
 */
STATIC SYSTIM  initialMilisecTime ;

/** ============================================================================
 *  @name   endMilisecTime
 *
 *  @desc   This varibale stores the value of  time in milliseconds.
 *          It is 64 bit size.
 *  ============================================================================
 */
STATIC SYSTIM  endMilisecTime ;

#endif /* if defined (MESSAGE_PROFILE) */


/** ============================================================================
 *  @func   kprintf
 *
 *  @desc   Extern declaration for kprintf function.
 *
 *  @arg    str
 *              String representation of the number.
 *  @arg    ...
 *              Variable arguments.
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ============================================================================
 */
#if !defined (DRX45X)
extern void kprintf (char* fp, ... ) ;
#endif /* !defined (DRX45X) */

/** ============================================================================
 *  @func   MESSAGEMULTIDSP_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_0Print (Char8 * str)
{
#if !defined (DRX45X)
   kprintf (str) ;
#else /* defined (DRX45X) */
   printf (str) ;
#endif /* !defined (DRX45X) */
}

/** ============================================================================
 *  @func   MESSAGEMULTIDSP_1Print
 *
 *  @desc   Print a message with two arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_1Print (Char8 * str, Uint32 arg)
{
#if !defined (DRX45X)
   kprintf (str, arg) ;
#else /* defined (DRX45X) */
   printf (str, arg) ;
#endif /* !defined (DRX45X) */
}

/** ============================================================================
 *  @func   MESSAGEMULTIDSP_2Print
 *
 *  @desc   Print a message with two arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_2Print (Char8 * str, Uint32 arg, Uint32 arg1)
{
#if !defined (DRX45X)
   kprintf (str, arg, arg1) ;
#else /* defined (DRX45X) */
   printf (str, arg, arg1) ;
#endif /* !defined (DRX45X) */
}

/** ============================================================================
 *  @func   MESSAGEMULTIDSP_Sleep
 *
 *  @desc   Sleeps for the specified number of microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_Sleep (Uint32 uSec)
{
    Uint32 waitMsecs = 1 ;

    /* Convert the usec into miliseconds. */
    if ((uSec / 1000) > 1) {
        waitMsecs = uSec / 1000 ;
    }

    dly_tsk((TMO)waitMsecs) ;
}

#if defined (MESSAGE_PROFILE)

/** ============================================================================
 *  @func   MESSAGE_ResetUsecTime
 *
 *  @desc   This function resets system time to zero.
 *
 *  @modif  initialUsecTime
 *  ============================================================================
 */
STATIC
Void
MESSAGEMULTIDSP_ResetUsecTime (Void)
{
    SYSTIM          resetMilisecTime ;
    DSP_STATUS      status              = DSP_SOK ;
    Int32           osStatus            = 0 ;

    resetMilisecTime.utime = 0 ;
    resetMilisecTime.ltime = 0 ;
    osStatus = set_tim (&resetMilisecTime) ;
    if (osStatus != 0) {
        status = DSP_EFAIL ;
        MESSAGEMULTIDSP_1Print ("System call set_tim returned error."
                                " Status = 0x%x", status) ;
    }
}

/** ============================================================================
 *  @func   MESSAGE_GetStartTime (Void)
 *
 *  @desc   This function records the start time in milliseconds.
 *
 *  @modif  startUsecTime
 *  ============================================================================
 */
EXPORT_API
Void
MESSAGEMULTIDSP_GetStartTime (Void)
{
    DSP_STATUS     status   = DSP_SOK ;
    ER             osStatus = 0       ;

    MESSAGEMULTIDSP_ResetUsecTime();
    osStatus = get_tim (&initialMilisecTime) ;
    if (osStatus != 0) {
        status = DSP_EFAIL ;
        MESSAGEMULTIDSP_1Print ("System call get_tim() returned error."
                                " Status = 0x%x", status) ;
    }
}

/** ============================================================================
 *  @func   MESSAGE_GetEndTime (Void)
 *
 *  @desc   This function records the end time in milliseconds.
 *
 *  @modif  endUsecTime
 *  ============================================================================
 */
EXPORT_API
Void
MESSAGEMULTIDSP_GetEndTime (Void)
{
    DSP_STATUS     status   = DSP_SOK ;
    ER             osStatus = 0       ;

    osStatus = get_tim (&endMilisecTime) ;
    if (osStatus != 0) {
        status = DSP_EFAIL ;
        MESSAGEMULTIDSP_1Print ("System call get_tim returned error."
                                " Status = 0x%x", status) ;
    }
}

/** ============================================================================
 *  @func   MESSAGE_GetProfileInfo
 *
 *  @desc   It converts a time variable to elapsed time structure.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
MESSAGEMULTIDSP_GetProfileInfo (Uint32 numIterations)
{

    SYSTIM         MilSecTimeTaken ;
    Real32         numSeconds ;
    Real32         roundTripTime    = 0 ;

    /*  If ltime overflows*/
    if( endMilisecTime.ltime  < initialMilisecTime.ltime ) {
        endMilisecTime.utime = endMilisecTime.utime - 1;
        endMilisecTime.ltime = 0xFFFFFFFF ;
    }

    MilSecTimeTaken.utime = (endMilisecTime.utime - initialMilisecTime.utime) ;
    MilSecTimeTaken.ltime = (endMilisecTime.ltime - initialMilisecTime.ltime) ;

    /* MilSecTimeTaken.ltime  is only considered  for profiling.since
     * MilSecTimeTaken.ltime is itself is huge value.
     * Ignored MilSecTimeTaken.utime.
     */
    numSeconds      = (Real32)((Real32) MilSecTimeTaken.ltime / 1000.0) ;

    MESSAGEMULTIDSP_1Print ("Transferring %d iterations took ", numIterations) ;
    MESSAGEMULTIDSP_1Print ("%d seconds ", numSeconds) ;
    MESSAGEMULTIDSP_1Print ("%d milliseconds.\n",
                            (Real32)(MilSecTimeTaken.ltime % 1000)) ;

    if (MilSecTimeTaken.ltime != 0) {
        roundTripTime =  (Real32)(  (Real32)(MilSecTimeTaken.ltime * 1000)
                               / (Real32)(numIterations)) ;

    }
    MESSAGEMULTIDSP_1Print ("RoundTrip Time for 1 message is %d "
                            "microseconds .\n", roundTripTime );

    return ;
}

#endif /* if defined (MESSAGE_PROFILE) */


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
