/** ============================================================================
 *  @file   mp_list_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/mp_list/PrOS/
 *
 *  @desc   OS specific implementation of functions used by the mp_list
 *          sample application.
 *
 *  @ver    1.65.01.06
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <itron.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>

/*  ----------------------------------- Application Header            */
#include <mp_list.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @const  SEM_NON_SIGNALLED
 *
 *  @desc   Non signalled semaphore value.
 *  ============================================================================
 */
#define SEM_NON_SIGNALLED    0

/** ============================================================================
 *  @const  SEM_SIGNALLED
 *
 *  @desc   Signalled semaphore value.
 *  ============================================================================
 */
#define SEM_SIGNALLED        1

/** ============================================================================
 *  @const  SEM_BINARY
 *
 *  @desc   Maximum count for binary type semaphore.
 *  ============================================================================
 */
#define SEM_BINARY           1

/** ============================================================================
 *  @const  SEM_COUNTING
 *
 *  @desc   Maximum count for counting type semaphore.
 *  ============================================================================
 */
#define SEM_COUNTING         0xFFFF

/** ============================================================================
 *  @name   MEM_mplId_sample
 *
 *  @desc   ID of memory pool created.
 *  ============================================================================
 */
STATIC ID MEM_mplId_sample ;

/** ============================================================================
 *  @macro  MPL_BUFF_SIZE
 *
 *  @desc   Memory pool size.
 *  ============================================================================
 */
#define  MPL_BUFF_SIZE  0x200000u

/** ============================================================================
 *  @name   mplbuff_sample
 *
 *  @desc   Start address of the variable-sized memory pool area.
 *  ============================================================================
 */
Uint8 mplbuff_sample [MPL_BUFF_SIZE] ;

/** ============================================================================
 *  @name   MP_LIST_SemObject
 *
 *  @desc   This object is used by various SEM API's.
 *
 *  @field  semId
 *              ID number of semaphore.
 *
 *  @see    None
 *  ============================================================================
 */
typedef struct MP_LIST_SemObject_tag {
    ID  semId ;
} MP_LIST_SemObject ;


/** ============================================================================
 *  @func   kprintf
 *
 *  @desc   Extern declaration for kprintf function.
 *
 *  @arg    str
 *              String representation of the number.
 *  @arg    ...
 *              Variable arguments.
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ============================================================================
 */
#if !defined (DRX45X)
extern void kprintf (char* fp, ... ) ;
#endif /* !defined (DRX45X) */

/** ============================================================================
 *  @func   MP_LIST_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MP_LIST_0Print (Char8 * str)
{
#if !defined (DRX45X)
   kprintf (str) ;
#else /* defined (DRX45X) */
   printf (str) ;
#endif /* !defined (DRX45X) */
}


/** ============================================================================
 *  @func   MP_LIST_1Print
 *
 *  @desc   Print a message with one arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MP_LIST_1Print (Char8 * str, Uint32 arg)
{
#if !defined (DRX45X)
   kprintf (str, arg) ;
#else /* defined (DRX45X) */
   printf (str, arg) ;
#endif /* !defined (DRX45X) */
}


/** ============================================================================
 *  @func   MP_LIST_CreateSem
 *
 *  @desc   This function creates a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MP_LIST_CreateSem (OUT Pvoid * semPtr)
{
    DSP_STATUS           status = DSP_SOK ;
    MP_LIST_SemObject * semObj = NULL ;
    ER_ID                erId   = E_OK ;
    T_CSEM               pk_csem ;
    T_CMPL     pk_cmpl           ;
    ER_ID      mplId             ;

    pk_cmpl.mplatr = TA_TPRI ;
    pk_cmpl.mplsz = MPL_BUFF_SIZE ;
    pk_cmpl.mpl = (VP) mplbuff_sample ;
    mplId = acre_mpl (&pk_cmpl) ;

    if (mplId > 0) {
        MEM_mplId_sample = mplId ;
    }
    else {
        status = DSP_EFAIL ;
    }

    if (DSP_SUCCEEDED (status)) {
        get_mpl (MEM_mplId_sample, sizeof (MP_LIST_SemObject), (VP *) &semObj) ;

        if (semObj != NULL) {
            pk_csem.sematr  = TA_TFIFO ;
            pk_csem.isemcnt = (UINT) SEM_NON_SIGNALLED   ;
            pk_csem.maxsem  = (UINT) SEM_COUNTING ;
            erId = acre_sem (&pk_csem) ;
            if (erId <= 0) {
                rel_mpl (MEM_mplId_sample, (VP) semObj) ;
                status = DSP_EFAIL ;
            }
            else {
                /* erId contains a valid semaphore id. */
                semObj->semId = erId ;
                *semPtr = (Pvoid) semObj ;
            }
        }
        else {
            *semPtr = NULL ;
            status = DSP_EMEMORY ;
        }
    }

    return status ;
}


/** ============================================================================
 *  @func   MP_LIST_DeleteSem
 *
 *  @desc   This function deletes a semaphore.
 *
 *  @arg    semHandle
 *              Pointer to the semaphore object to be deleted.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MP_LIST_DeleteSem (IN Pvoid semHandle)
{
    DSP_STATUS           status   = DSP_SOK ;
    MP_LIST_SemObject * semObj   = semHandle ;
    ER                   erStatus = E_OK ;

    erStatus = del_sem (semObj->semId) ;
    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    rel_mpl (MEM_mplId_sample, (VP) semObj) ;

    return status ;
}


/** ============================================================================
 *  @func   MP_LIST_WaitSem
 *
 *  @desc   This function waits on a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MP_LIST_WaitSem (IN Pvoid semHandle)
{
    DSP_STATUS           status   = DSP_SOK ;
    MP_LIST_SemObject * semObj   = semHandle ;
    ER                   erStatus = E_OK ;

    erStatus = wai_sem (semObj->semId) ;
    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    return status ;
}


/** ============================================================================
 *  @func   MP_LIST_PostSem
 *
 *  @desc   This function posts a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MP_LIST_PostSem (IN Pvoid semHandle)
{
    DSP_STATUS           status   = DSP_SOK ;
    MP_LIST_SemObject * semObj   = semHandle ;
    ER                   erStatus = E_OK ;
    BOOL                 state ;

    /* get the current context. */
    state = sns_ctx () ;

    if (state == FALSE) {
        erStatus = sig_sem (semObj->semId) ;
    }
    else {
        erStatus = isig_sem (semObj->semId) ;
    }

    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    return status ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
